from __future__ import unicode_literals

from ..models import DeletedDocument, Document
from ..permissions import (
    permission_document_delete, permission_document_restore,
    permission_document_trash, permission_document_view
)

from .base import GenericDocumentViewTestCase
from .mixins import TrashedDocumentViewTestMixin


class TrashedDocumentViewTestCase(
    TrashedDocumentViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_trash_get_view_no_permissions(self):
        document_count = Document.objects.count()

        response = self._request_document_trash_get_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), document_count)

    def test_document_trash_get_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_trash
        )

        document_count = Document.objects.count()

        response = self._request_document_trash_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Document.objects.count(), document_count)

    def test_document_trash_post_view_no_permissions(self):
        response = self._request_document_trash_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 1)

    def test_document_trash_post_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_trash
        )

        response = self._request_document_trash_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DeletedDocument.objects.count(), 1)
        self.assertEqual(Document.objects.count(), 0)

    def test_trashed_document_restore_get_view_no_permission(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)

        document_count = Document.objects.count()

        response = self._request_trashed_document_restore_get_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), document_count)

    def test_trashed_document_restore_get_view_with_access(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)

        self.grant_access(
            obj=self.test_document, permission=permission_document_restore
        )

        document_count = Document.objects.count()

        response = self._request_trashed_document_restore_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(Document.objects.count(), document_count)

    def test_trashed_document_restore_post_view_no_permission(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)

        response = self._request_trashed_document_restore_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(DeletedDocument.objects.count(), 1)
        self.assertEqual(Document.objects.count(), 0)

    def test_trashed_document_restore_post_view_with_access(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)

        self.grant_access(
            obj=self.test_document, permission=permission_document_restore
        )

        response = self._request_trashed_document_restore_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 1)

    def test_trashed_document_delete_get_view_no_permissions(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)
        self.assertEqual(DeletedDocument.objects.count(), 1)

        trashed_document_count = DeletedDocument.objects.count()

        response = self._request_trashed_document_delete_get_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            DeletedDocument.objects.count(), trashed_document_count
        )

    def test_trashed_document_delete_get_view_with_access(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)
        self.assertEqual(DeletedDocument.objects.count(), 1)

        self.grant_access(
            obj=self.test_document, permission=permission_document_delete
        )

        trashed_document_count = DeletedDocument.objects.count()

        response = self._request_trashed_document_delete_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            DeletedDocument.objects.count(), trashed_document_count
        )

    def test_trashed_document_delete_post_view_no_permissions(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)
        self.assertEqual(DeletedDocument.objects.count(), 1)

        response = self._request_trashed_document_delete_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Document.objects.count(), 0)
        self.assertEqual(DeletedDocument.objects.count(), 1)

    def test_trashed_document_delete_post_view_with_access(self):
        self.test_document.delete()
        self.assertEqual(Document.objects.count(), 0)
        self.assertEqual(DeletedDocument.objects.count(), 1)

        self.grant_access(
            obj=self.test_document, permission=permission_document_delete
        )

        response = self._request_trashed_document_delete_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 0)

    def test_trashed_document_list_view_no_permissions(self):
        self.test_document.delete()

        response = self._request_trashed_document_list_view()
        self.assertNotContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_trashed_document_list_view_with_access(self):
        self.test_document.delete()

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_trashed_document_list_view()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIResolvedSmartLinkView, APIResolvedSmartLinkDocumentListView,
    APIResolvedSmartLinkListView, APISmartLinkListView, APISmartLinkView,
    APISmartLinkConditionListView, APISmartLinkConditionView
)
from .views import (
    DocumentSmartLinkListView, DocumentTypeSmartLinksView,
    ResolvedSmartLinkView, SetupSmartLinkDocumentTypesView,
    SmartLinkConditionListView, SmartLinkConditionCreateView,
    SmartLinkConditionEditView, SmartLinkConditionDeleteView,
    SmartLinkCreateView, SmartLinkDeleteView, SmartLinkEditView,
    SmartLinkListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<pk>\d+)/smart_links/$',
        view=DocumentSmartLinkListView.as_view(),
        name='smart_link_instances_for_document'
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/smart_links/(?P<smart_link_pk>\d+)/$',
        view=ResolvedSmartLinkView.as_view(), name='smart_link_instance_view'
    ),
    url(
        regex=r'^document_types/(?P<pk>\d+)/smart_links/$',
        view=DocumentTypeSmartLinksView.as_view(),
        name='document_type_smart_links'
    ),
    url(
        regex=r'^smart_links/$', view=SmartLinkListView.as_view(),
        name='smart_link_list'
    ),
    url(
        regex=r'^smart_links/create/$', view=SmartLinkCreateView.as_view(),
        name='smart_link_create'
    ),
    url(
        regex=r'^smart_links/(?P<pk>\d+)/delete/$',
        view=SmartLinkDeleteView.as_view(), name='smart_link_delete'
    ),
    url(
        regex=r'^smart_links/(?P<pk>\d+)/edit/$', view=SmartLinkEditView.as_view(),
        name='smart_link_edit'
    ),
    url(
        regex=r'^smart_links/(?P<pk>\d+)/document_types/$',
        view=SetupSmartLinkDocumentTypesView.as_view(),
        name='smart_link_document_types'
    ),
    url(
        regex=r'^smart_links/(?P<pk>\d+)/conditions/$',
        view=SmartLinkConditionListView.as_view(),
        name='smart_link_condition_list'
    ),
    url(
        regex=r'^smart_links/(?P<pk>\d+)/conditions/create/$',
        view=SmartLinkConditionCreateView.as_view(),
        name='smart_link_condition_create'
    ),
    url(
        regex=r'^smart_links/conditions/(?P<pk>\d+)/delete/$',
        view=SmartLinkConditionDeleteView.as_view(),
        name='smart_link_condition_delete'
    ),
    url(
        regex=r'^smart_links/conditions/(?P<pk>\d+)/edit/$',
        view=SmartLinkConditionEditView.as_view(),
        name='smart_link_condition_edit'
    ),
]

api_urls = [
    url(
        regex=r'^smart_links/$', view=APISmartLinkListView.as_view(),
        name='smartlink-list'
    ),
    url(
        regex=r'^smart_links/(?P<pk>[0-9]+)/$',
        view=APISmartLinkView.as_view(), name='smartlink-detail'
    ),
    url(
        regex=r'^smart_links/(?P<pk>[0-9]+)/conditions/$',
        view=APISmartLinkConditionListView.as_view(),
        name='smartlinkcondition-list'
    ),
    url(
        regex=r'^smart_links/(?P<pk>[0-9]+)/conditions/(?P<condition_pk>[0-9]+)/$',
        view=APISmartLinkConditionView.as_view(),
        name='smartlinkcondition-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/resolved_smart_links/$',
        view=APIResolvedSmartLinkListView.as_view(),
        name='resolvedsmartlink-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/resolved_smart_links/(?P<smart_link_pk>[0-9]+)/$',
        view=APIResolvedSmartLinkView.as_view(),
        name='resolvedsmartlink-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/resolved_smart_links/(?P<smart_link_pk>[0-9]+)/documents/$',
        view=APIResolvedSmartLinkDocumentListView.as_view(),
        name='resolvedsmartlinkdocument-list'
    ),
]



from __future__ import unicode_literals

from django.db import migrations, models
from django.core.files.storage import FileSystemStorage

import mayan.apps.document_signatures.models


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0033_auto_20160325_0052'),
        ('document_signatures', '0002_auto_20150608_1902'),
    ]

    operations = [
        migrations.CreateModel(
            name='SignatureBaseModel',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'date', models.DateField(
                        null=True, verbose_name='Date signed', blank=True
                    )
                ),
                (
                    'key_id', models.CharField(
                        max_length=40, verbose_name='Key ID'
                    )
                ),
                (
                    'signature_id', models.CharField(
                        max_length=64, null=True, verbose_name='Signature ID',
                        blank=True
                    )
                ),
                (
                    'public_key_fingerprint', models.CharField(
                        verbose_name='Public key fingerprint', unique=True,
                        max_length=40, editable=False
                    )
                ),
            ],
            options={
                'verbose_name': 'Document version signature',
                'verbose_name_plural': 'Document version signatures',
            },
        ),
        migrations.RemoveField(
            model_name='documentversionsignature',
            name='has_embedded_signature',
        ),
        migrations.AddField(
            model_name='documentversionsignature',
            name='date',
            field=models.DateField(
                null=True, verbose_name='Date signed', blank=True
            ),
        ),
        migrations.AddField(
            model_name='documentversionsignature',
            name='signature_id',
            field=models.CharField(
                max_length=64, null=True, verbose_name='Signature ID',
                blank=True
            ),
        ),
        migrations.AlterField(
            model_name='documentversionsignature',
            name='document_version',
            field=models.ForeignKey(
                related_name='signature', editable=False,
                to='documents.DocumentVersion', verbose_name='Document version'
            ),
        ),
        migrations.CreateModel(
            name='DetachedSignature',
            fields=[
                (
                    'signaturebasemodel_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False,
                        to='document_signatures.SignatureBaseModel'
                    )
                ),
                (
                    'signature_file', models.FileField(
                        storage=FileSystemStorage(),
                        upload_to=mayan.apps.document_signatures.models.upload_to,
                        null=True, verbose_name='Signature file', blank=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Document version detached signature',
                'verbose_name_plural': 'Document version detached signatures',
            },
            bases=('document_signatures.signaturebasemodel',),
        ),
        migrations.CreateModel(
            name='EmbeddedSignature',
            fields=[
                (
                    'signaturebasemodel_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False,
                        to='document_signatures.SignatureBaseModel'
                    )
                ),
            ],
            options={
                'verbose_name': 'Document version embedded signature',
                'verbose_name_plural': 'Document version embedded signatures',
            },
            bases=('document_signatures.signaturebasemodel',),
        ),
        migrations.AddField(
            model_name='signaturebasemodel',
            name='document_version',
            field=models.ForeignKey(
                related_name='signaturebasemodel', editable=False,
                to='documents.DocumentVersion', verbose_name='Document version'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.permissions import permission_document_type_edit
from mayan.apps.navigation.classes import Link

from .permissions import (
    permission_document_metadata_add, permission_document_metadata_edit,
    permission_document_metadata_remove, permission_document_metadata_view,
    permission_metadata_type_create, permission_metadata_type_delete,
    permission_metadata_type_edit, permission_metadata_type_view
)

link_metadata_add = Link(
    args='object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_add',
    permissions=(permission_document_metadata_add,), text=_('Add metadata'),
    view='metadata:metadata_add',
)
link_metadata_edit = Link(
    args='object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_edit',
    permissions=(permission_document_metadata_edit,),
    text=_('Edit metadata'), view='metadata:metadata_edit'
)
link_metadata_multiple_add = Link(
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_add',
    text=_('Add metadata'), view='metadata:metadata_multiple_add'
)
link_metadata_multiple_edit = Link(
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_edit',
    text=_('Edit metadata'), view='metadata:metadata_multiple_edit'
)
link_metadata_multiple_remove = Link(
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_remove',
    text=_('Remove metadata'), view='metadata:metadata_multiple_remove'
)
link_metadata_remove = Link(
    args='object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_remove',
    permissions=(permission_document_metadata_remove,),
    text=_('Remove metadata'), view='metadata:metadata_remove',
)
link_metadata_view = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_document_metadata_view',
    permissions=(permission_document_metadata_view,), text=_('Metadata'),
    view='metadata:metadata_view',
)
link_setup_document_type_metadata_types = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_document_type_metadata_type_list',
    permissions=(permission_document_type_edit,),
    text=_('Metadata types'), view='metadata:setup_document_type_metadata_types',
)
link_setup_metadata_type_document_types = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_metadata_type_document_type_list',
    permissions=(permission_document_type_edit,),
    text=_('Document types'), view='metadata:setup_metadata_type_document_types',
)
link_setup_metadata_type_create = Link(
    icon_class_path='mayan.apps.metadata.icons.icon_metadata_type_create',
    permissions=(permission_metadata_type_create,), text=_('Create new'),
    view='metadata:setup_metadata_type_create'
)
link_setup_metadata_type_delete = Link(
    args='object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_metadata_type_delete',
    permissions=(permission_metadata_type_delete,),
    tags='dangerous', text=_('Delete'), view='metadata:setup_metadata_type_delete',
)
link_setup_metadata_type_edit = Link(
    args='object.pk',
    icon_class_path='mayan.apps.metadata.icons.icon_metadata_type_edit',
    permissions=(permission_metadata_type_edit,),
    text=_('Edit'), view='metadata:setup_metadata_type_edit'
)
link_setup_metadata_type_list = Link(
    icon_class_path='mayan.apps.metadata.icons.icon_metadata_type_list',
    permissions=(permission_metadata_type_view,),
    text=_('Metadata types'), view='metadata:setup_metadata_type_list'
)



from __future__ import unicode_literals

from django.template import Library

from ..models import AutoAdminSingleton

register = Library()


@register.inclusion_tag('autoadmin/credentials.html')
def autoadmin_partial():
    try:
        return {'autoadmin_properties': AutoAdminSingleton.objects.get()}
    except AutoAdminSingleton.DoesNotExist:
        return {'autoadmin_properties': None}


@register.simple_tag(takes_context=True)
def autoadmin_properties(context):
    try:
        context['autoadmin_properties'] = AutoAdminSingleton.objects.get()
    except AutoAdminSingleton.DoesNotExist:
        context['autoadmin_properties'] = None

    return ''



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0005_auto_20150617_0358'),
        ('ocr', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentPageContent',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'content', models.TextField(
                        verbose_name='Content', blank=True
                    )
                ),
                (
                    'document_page', models.OneToOneField(
                        related_name='ocr_content',
                        verbose_name='Document page',
                        to='documents.DocumentPage'
                    )
                ),
            ],
            options={
                'verbose_name': 'Document page content',
                'verbose_name_plural': 'Document pages contents',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model


class UserForm(forms.ModelForm):
    """
    Form used to edit a user's mininal fields by the user himself
    """
    class Meta:
        fields = (
            'username', 'first_name', 'last_name', 'email', 'is_active',
        )
        model = get_user_model()



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Smart links'), name='linking')

permission_smart_link_create = namespace.add_permission(
    label=_('Create new smart links'), name='smart_link_create'
)
permission_smart_link_delete = namespace.add_permission(
    label=_('Delete smart links'), name='smart_link_delete'
)
permission_smart_link_edit = namespace.add_permission(
    label=_('Edit smart links'), name='smart_link_edit'
)
permission_smart_link_view = namespace.add_permission(
    label=_('View existing smart links'), name='smart_link_view'
)



from __future__ import absolute_import, unicode_literals

from mayan.apps.rest_api import generics

from .models import Message
from .permissions import (
    permission_message_create, permission_message_delete,
    permission_message_edit, permission_message_view
)
from .serializers import MessageSerializer


class APIMessageListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the messages.
    post: Create a new message.
    """
    mayan_object_permissions = {'GET': (permission_message_view,)}
    mayan_view_permissions = {'POST': (permission_message_create,)}
    queryset = Message.objects.all()
    serializer_class = MessageSerializer


class APIMessageView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected message.
    get: Return the details of the selected message.
    patch: Edit the selected message.
    put: Edit the selected message.
    """
    mayan_object_permissions = {
        'DELETE': (permission_message_delete,),
        'GET': (permission_message_view,),
        'PATCH': (permission_message_edit,),
        'PUT': (permission_message_edit,)
    }
    queryset = Message.objects.all()
    serializer_class = MessageSerializer



from __future__ import absolute_import, unicode_literals

from rest_framework import status

from mayan.apps.permissions.tests.literals import TEST_ROLE_LABEL
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models import AccessControlList
from ..permissions import permission_acl_edit, permission_acl_view

from .mixins import ACLTestMixin


class ACLAPITestCase(ACLTestMixin, BaseAPITestCase):
    auto_create_test_object = True

    def _request_acl_create_api_view(self, extra_data=None):
        data = {'role_pk': self.test_role.pk}

        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='rest_api:accesscontrollist-list',
            kwargs=self.test_content_object_view_kwargs, data=data
        )

    def test_acl_create_api_api_view_with_access(self):
        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        response = self._request_acl_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        pk = response.data['id']

        test_object_acl = self.test_object.acls.get(pk=pk)
        self.assertEqual(
            test_object_acl.role, self.test_role
        )
        self.assertEqual(
            test_object_acl.content_object, self.test_object
        )
        self.assertEqual(
            test_object_acl.permissions.count(), 0
        )

    def test_acl_create_post_api_extra_data_view_with_access(self):
        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        response = self._request_acl_create_api_view(
            extra_data={'permissions_pk_list': permission_acl_view.pk}
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        pk = response.data['id']

        test_object_acl = self.test_object.acls.get(pk=pk)

        self.assertEqual(
            test_object_acl.content_object, self.test_object
        )
        self.assertEqual(
            test_object_acl.role, self.test_role
        )
        self.assertEqual(
            test_object_acl.permissions.first(),
            permission_acl_view.stored_permission
        )

    def _request_test_acl_delete_api_view(self):
        return self.delete(
            viewname='rest_api:accesscontrollist-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }
        )

    def test_acl_delete_api_view_with_access(self):
        self._create_test_acl()

        self.grant_access(self.test_object, permission=permission_acl_edit)

        acl_count = AccessControlList.objects.count()

        response = self._request_test_acl_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(AccessControlList.objects.count(), acl_count - 1)

    def _request_test_acl_permission_delete_api_view(self):
        return self.delete(
            viewname='rest_api:accesscontrollist-permission-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk,
                'permission_pk': self.test_permission.stored_permission.pk
            }
        )

    def test_acl_permission_delete_view_with_access(self):
        self._create_test_acl()
        self.test_acl.permissions.add(self.test_permission.stored_permission)

        self.grant_access(obj=self.test_object, permission=permission_acl_edit)

        response = self._request_test_acl_permission_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(self.test_acl.permissions.count(), 0)

    def test_acl_detail_api_view_with_access(self):
        self._create_test_acl()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_view
        )

        response = self._request_test_acl_detail_api_view()

        self.assertEqual(
            response.data['content_type']['app_label'],
            self.test_object_content_type.app_label
        )
        self.assertEqual(
            response.data['role']['label'], TEST_ROLE_LABEL
        )

    def _request_test_acl_permission_detail_api_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-permission-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk,
                'permission_pk': self.test_acl.permissions.first().pk
            }
        )

    def test_acl_permission_detail_api_view_with_access(self):
        self._create_test_acl()
        self.test_acl.permissions.add(self.test_permission.stored_permission)

        self.grant_access(
            obj=self.test_object, permission=permission_acl_view
        )

        response = self._request_test_acl_permission_detail_api_view()
        self.assertEqual(
            response.data['pk'], self.test_permission.pk
        )

    def test_acl_list_api_view_with_access(self):
        self._create_test_acl()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_view
        )

        response = self.get(
            viewname='rest_api:accesscontrollist-list', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertContains(
            response=response, text=self.test_object_content_type.app_label,
            status_code=200
        )
        self.assertContains(
            response=response, text=self.test_acl.role.label,
            status_code=200
        )

    def _request_test_acl_detail_api_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }
        )

    def _request_test_acl_permission_list_api_get_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-permission-list', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }
        )

    def test_acl_permission_list_api_get_view_with_access(self):
        self._create_test_acl()
        self.test_acl.permissions.add(self.test_permission.stored_permission)

        self.grant_access(
            obj=self.test_object, permission=permission_acl_view
        )

        response = self._request_test_acl_permission_list_api_get_view()
        self.assertEqual(
            response.data['results'][0]['pk'],
            self.test_permission.pk
        )

    def _request_acl_permssion_list_api_post_view(self):
        return self.post(
            viewname='rest_api:accesscontrollist-permission-list', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }, data={'permission_pk': self.test_permission.pk}
        )

    def test_acl_permission_list_api_post_view_with_access(self):
        self._create_test_acl()

        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        response = self._request_acl_permssion_list_api_post_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertTrue(
            self.test_permission.stored_permission in self.test_acl.permissions.all()
        )



from __future__ import unicode_literals

import time

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import TEST_SMALL_DOCUMENT_PATH
from mayan.apps.documents.tests.mixins import DocumentTestMixin

from ..exceptions import (
    DocumentAlreadyCheckedOut, DocumentNotCheckedOut,
    NewDocumentVersionNotAllowed
)
from ..models import DocumentCheckout, NewVersionBlock

from .mixins import DocumentCheckoutTestMixin


class DocumentCheckoutTestCase(
    DocumentCheckoutTestMixin, GenericDocumentTestCase
):
    def test_document_check_out(self):
        self._check_out_test_document()

        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_document
            )
        )

    def test_document_check_in(self):
        self._check_out_test_document()

        self.test_document.check_in()

        self.assertFalse(self.test_document.is_checked_out())
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_document
            )
        )

    def test_document_double_check_out(self):
        self._create_test_case_superuser()
        self._check_out_test_document()

        with self.assertRaises(DocumentAlreadyCheckedOut):
            DocumentCheckout.objects.check_out_document(
                document=self.test_document,
                expiration_datetime=self._check_out_expiration_datetime,
                user=self._test_case_superuser,
                block_new_version=True
            )

    def test_document_check_in_without_check_out(self):
        with self.assertRaises(DocumentNotCheckedOut):
            self.test_document.check_in()

    def test_document_auto_check_in(self):
        self._check_out_test_document()

        # Ensure we wait from longer than the document check out expiration
        time.sleep(self._test_document_check_out_seconds + 0.1)

        DocumentCheckout.objects.check_in_expired_check_outs()

        self.assertFalse(self.test_document.is_checked_out())


class NewVersionBlockTestCase(
    DocumentCheckoutTestMixin, DocumentTestMixin, BaseTestCase
):
    def test_blocking(self):
        NewVersionBlock.objects.block(document=self.test_document)

        self.assertEqual(NewVersionBlock.objects.count(), 1)
        self.assertEqual(
            NewVersionBlock.objects.first().document, self.test_document
        )

    def test_blocking_new_versions(self):
        # Silence unrelated logging
        self._silence_logger(name='mayan.apps.documents.models')

        NewVersionBlock.objects.block(document=self.test_document)

        with self.assertRaises(NewDocumentVersionNotAllowed):
            with open(TEST_SMALL_DOCUMENT_PATH, mode='rb') as file_object:
                self.test_document.new_version(file_object=file_object)

    def test_unblocking(self):
        NewVersionBlock.objects.create(document=self.test_document)

        NewVersionBlock.objects.unblock(document=self.test_document)

        self.assertEqual(NewVersionBlock.objects.count(), 0)

    def test_is_blocked(self):
        NewVersionBlock.objects.create(document=self.test_document)

        self.assertTrue(
            NewVersionBlock.objects.is_blocked(document=self.test_document)
        )

        NewVersionBlock.objects.all().delete()

        self.assertFalse(
            NewVersionBlock.objects.is_blocked(document=self.test_document)
        )

    def test_version_creation_blocking(self):
        # Silence unrelated logging
        self._silence_logger(name='mayan.apps.documents.models')

        self._create_test_case_superuser()

        self._check_out_test_document()

        with self.assertRaises(NewDocumentVersionNotAllowed):
            with open(TEST_SMALL_DOCUMENT_PATH, mode='rb') as file_object:
                self.test_document.new_version(file_object=file_object)



from __future__ import unicode_literals


class BaseCommonException(Exception):
    """
    Base exception for the common app
    """
    pass


class ActionError(BaseCommonException):
    """
    Raise by the MultiActionConfirmView to announce when the object action
    failed for one or more items.  This exception doesn't stop the iteration,
    it is used to announce that one item in the queryset failed to process.
    """


class CompressionFileError(BaseCommonException):
    """
    Base exception for file decompression class
    """
    pass


class NoMIMETypeMatch(CompressionFileError):
    """
    There is no decompressor registered for the specified MIME type
    """
    pass


class NPMException(BaseCommonException):
    """Base exception for the NPM registry client"""


class NPMPackgeIntegrityError(NPMException):
    """Hash mismatch exception"""



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('motd', '0002_auto_20160313_0340'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messageoftheday',
            name='message',
            field=models.TextField(verbose_name='Message'),
        ),
    ]



from __future__ import unicode_literals

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.document_states.tests.base import ActionTestCase
from mayan.apps.document_states.tests.mixins import WorkflowTestMixin

from ..models import Tag
from ..workflow_actions import AttachTagAction, RemoveTagAction

from .mixins import TagTestMixin


class TagActionTestCase(TagTestMixin, ActionTestCase):
    def setUp(self):
        super(TagActionTestCase, self).setUp()
        self._create_test_tag()

    def test_tag_attach_action(self):
        action = AttachTagAction(form_data={'tags': Tag.objects.all()})
        action.execute(context={'document': self.test_document})

        self.assertEqual(self.test_tag.documents.count(), 1)
        self.assertTrue(self.test_document in self.test_tag.documents.all())

    def test_tag_remove_action(self):
        self.test_tag.attach_to(document=self.test_document)

        action = RemoveTagAction(form_data={'tags': Tag.objects.all()})
        action.execute(context={'document': self.test_document})

        self.assertEqual(self.test_tag.documents.count(), 0)


class TagActionViewTestCase(WorkflowTestMixin, GenericViewTestCase):
    def test_tag_attach_action_create_view(self):
        self._create_test_workflow()
        self._create_test_workflow_state()

        response = self.get(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'pk': self.test_workflow_state.pk,
                'class_path': 'mayan.apps.tags.workflow_actions.AttachTagAction'
            }
        )

        self.assertEqual(response.status_code, 200)

    def test_tag_remove_action_create_view(self):
        self._create_test_workflow()
        self._create_test_workflow_state()

        response = self.get(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'pk': self.test_workflow_state.pk,
                'class_path': 'mayan.apps.tags.workflow_actions.RemoveTagAction'
            }
        )

        self.assertEqual(response.status_code, 200)



from __future__ import unicode_literals

from ..models import FavoriteDocument
from ..permissions import permission_document_view

from .base import GenericDocumentViewTestCase


class FavoriteDocumentsTestMixin(object):
    def _request_document_add_to_favorites_view(self):
        return self.post(
            viewname='documents:document_add_to_favorites',
            kwargs={'pk': self.test_document.pk}
        )

    def _document_add_to_favorites(self):
        FavoriteDocument.objects.add_for_user(
            document=self.test_document, user=self._test_case_user
        )

    def _request_document_list_favorites(self):
        return self.get(
            viewname='documents:document_list_favorites',
        )

    def _request_document_remove_from_favorites(self):
        return self.post(
            viewname='documents:document_remove_from_favorites',
            kwargs={'pk': self.test_document.pk}
        )


class FavoriteDocumentsTestCase(
    FavoriteDocumentsTestMixin, GenericDocumentViewTestCase
):

    def test_document_add_to_favorites_view_no_permission(self):
        response = self._request_document_add_to_favorites_view()
        self.assertEqual(response.status_code, 404)
        self.assertEqual(FavoriteDocument.objects.count(), 0)

    def test_document_add_to_favorites_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        response = self._request_document_add_to_favorites_view()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(FavoriteDocument.objects.count(), 1)

    def test_document_list_favorites_view_no_permission(self):
        self._document_add_to_favorites()
        response = self._request_document_list_favorites()
        self.assertNotContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_list_favorites_view_with_access(self):
        self._document_add_to_favorites()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        response = self._request_document_list_favorites()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_remove_from_favorites_view_no_permission(self):
        self._document_add_to_favorites()
        response = self._request_document_remove_from_favorites()
        self.assertEqual(response.status_code, 404)
        self.assertEqual(FavoriteDocument.objects.count(), 1)

    def test_document_remove_from_favorites_view_with_access(self):
        self._document_add_to_favorites()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        response = self._request_document_remove_from_favorites()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(FavoriteDocument.objects.count(), 0)



from __future__ import absolute_import, unicode_literals

from django.template import RequestContext
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.generics import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectDetailView,
    SingleObjectEditView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.documents.models import Document

from .forms import DocumentCommentDetailForm
from .icons import icon_comments_for_document
from .links import link_comment_add
from .models import Comment
from .permissions import (
    permission_document_comment_create, permission_document_comment_delete,
    permission_document_comment_edit, permission_document_comment_view
)


class DocumentCommentCreateView(ExternalObjectMixin, SingleObjectCreateView):
    external_object_class = Document
    external_object_permission = permission_document_comment_create
    external_object_pk_url_kwarg = 'pk'
    fields = ('comment',)

    def get_extra_context(self):
        return {
            'object': self.external_object,
            'title': _('Add comment to document: %s') % self.external_object,
        }

    def get_instance_extra_data(self):
        return {
            'document': self.external_object, 'user': self.request.user,
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='comments:comments_for_document', kwargs={
                'pk': self.kwargs['pk']
            }
        )

    def get_queryset(self):
        return self.external_object.comments.all()

    def get_save_extra_data(self):
        return {
            '_user': self.request.user,
        }


class DocumentCommentDeleteView(SingleObjectDeleteView):
    model = Comment
    pk_url_kwarg = 'pk'
    object_permission = permission_document_comment_delete

    def get_delete_extra_data(self):
        return {'_user': self.request.user}

    def get_extra_context(self):
        return {
            'comment': self.object,
            'document': self.object.document,
            'navigation_object_list': ('document', 'comment'),
            'title': _('Delete comment: %s?') % self.object,
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='comments:comments_for_document', kwargs={
                'pk': self.object.document.pk
            }
        )


class DocumentCommentDetailView(SingleObjectDetailView):
    form_class = DocumentCommentDetailForm
    model = Comment
    pk_url_kwarg = 'pk'
    object_permission = permission_document_comment_view

    def get_extra_context(self):
        return {
            'comment': self.object,
            'document': self.object.document,
            'navigation_object_list': ('document', 'comment'),
            'title': _('Details for comment: %s?') % self.object,
        }


class DocumentCommentEditView(SingleObjectEditView):
    fields = ('comment',)
    model = Comment
    pk_url_kwarg = 'pk'
    object_permission = permission_document_comment_edit

    def get_save_extra_data(self):
        return {'_user': self.request.user}

    def get_extra_context(self):
        return {
            'comment': self.object,
            'document': self.object.document,
            'navigation_object_list': ('document', 'comment'),
            'title': _('Edit comment: %s?') % self.object,
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='comments:comments_for_document', kwargs={
                'pk': self.object.document.pk
            }
        )


class DocumentCommentListView(ExternalObjectMixin, SingleObjectListView):
    external_object_class = Document
    external_object_permission = permission_document_comment_view
    external_object_pk_url_kwarg = 'pk'

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_comments_for_document,
            'no_results_external_link': link_comment_add.resolve(
                RequestContext(self.request, {'object': self.external_object})
            ),
            'no_results_text': _(
                'Document comments are timestamped text entries from users. '
                'They are great for collaboration.'
            ),
            'no_results_title': _('There are no comments'),
            'object': self.external_object,
            'title': _('Comments for document: %s') % self.external_object,
        }

    def get_source_queryset(self):
        return self.external_object.comments.all()



from __future__ import unicode_literals

import datetime

from django.utils.timezone import now

from mayan.apps.common.literals import TIME_DELTA_UNIT_DAYS
from mayan.apps.common.tests.utils import as_id_list

from ..models import DocumentCheckout


class DocumentCheckoutsAPIViewTestMixin(object):
    def _request_test_document_check_out_create_api_view(self):
        return self.post(
            viewname='rest_api:checkout-document-list', data={
                'document_pk': self.test_document.pk,
                'expiration_datetime': '2099-01-01T12:00'
            }
        )

    def _request_test_document_check_out_delete_api_view(self):
        return self.delete(
            viewname='rest_api:checkedout-document-view',
            kwargs={'pk': self.test_check_out.pk}
        )

    def _request_test_document_check_out_detail_api_view(self):
        return self.get(
            viewname='rest_api:checkedout-document-view',
            kwargs={'pk': self.test_check_out.pk}
        )

    def _request_test_document_check_out_list_api_view(self):
        return self.get(viewname='rest_api:checkout-document-list')


class DocumentCheckoutTestMixin(object):
    _test_document_check_out_seconds = 0.1

    def _check_out_test_document(self, document=None, user=None):
        if not document:
            document = self.test_document

        if not user:
            user = self._test_case_user

        self._check_out_expiration_datetime = now() + datetime.timedelta(
            seconds=self._test_document_check_out_seconds
        )

        self.test_check_out = DocumentCheckout.objects.check_out_document(
            block_new_version=True, document=document,
            expiration_datetime=self._check_out_expiration_datetime,
            user=user
        )


class DocumentCheckoutViewTestMixin(object):
    def _request_test_document_check_in_get_view(self):
        return self.get(
            viewname='checkouts:check_in_document', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_document_check_in_post_view(self):
        return self.post(
            viewname='checkouts:check_in_document', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_document_multiple_check_in_post_view(self):
        return self.post(
            viewname='checkouts:check_in_document_multiple', data={
                'id_list': as_id_list(items=self.test_documents)
            }
        )

    def _request_test_document_check_out_get_view(self):
        return self.get(
            viewname='checkouts:check_out_document', kwargs={
                'pk': self.test_document.pk
            },
        )

    def _request_test_document_check_out_post_view(self):
        return self.post(
            viewname='checkouts:check_out_document', kwargs={
                'pk': self.test_document.pk
            }, data={
                'expiration_datetime_unit': TIME_DELTA_UNIT_DAYS,
                'expiration_datetime_amount': 99,
                'block_new_version': True
            }
        )

    def _request_test_document_multiple_check_out_post_view(self):
        return self.post(
            viewname='checkouts:check_out_document_multiple', data={
                'block_new_version': True,
                'expiration_datetime_unit': TIME_DELTA_UNIT_DAYS,
                'expiration_datetime_amount': 99,
                'id_list': as_id_list(items=self.test_documents)
            }
        )

    def _request_test_document_check_out_detail_view(self):
        return self.get(
            viewname='checkouts:check_out_info', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_document_check_out_list_view(self):
        return self.get(viewname='checkouts:check_out_list')



from __future__ import unicode_literals

import logging

from django.contrib.auth.models import Group
from django.db import models, transaction
from django.urls import reverse
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from mayan.apps.user_management.events import event_group_edited

from .classes import Permission
from .events import event_role_created, event_role_edited
from .managers import RoleManager, StoredPermissionManager

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class StoredPermission(models.Model):
    """
    This model is the counterpart of the permissions.classes.Permission
    class. Allows storing a database counterpart of a permission class.
    It is used to store the permissions help by a role or in an ACL.
    """
    namespace = models.CharField(max_length=64, verbose_name=_('Namespace'))
    name = models.CharField(max_length=64, verbose_name=_('Name'))

    objects = StoredPermissionManager()

    class Meta:
        ordering = ('namespace',)
        unique_together = ('namespace', 'name')
        verbose_name = _('Permission')
        verbose_name_plural = _('Permissions')

    def __str__(self):
        try:
            return force_text(self.volatile_permission)
        except KeyError:
            return self.name

    @property
    def volatile_permission_id(self):
        """
        Return the identifier of the real permission class represented by
        this model instance.
        """
        return '{}.{}'.format(self.namespace, self.name)

    @property
    def volatile_permission(self):
        """
        Returns the real class of the permission represented by this model
        instance.
        """
        return Permission.get(
            pk=self.volatile_permission_id, class_only=True
        )

    def natural_key(self):
        return (self.namespace, self.name)

    def user_has_this(self, user):
        """
        Helper method to check if a user has been granted this permission.
        The check is done sequentially over all of the user's groups and
        roles. The check is interrupted at the first positive result.
        The check always returns True for superusers or staff users.
        """
        if user.is_superuser or user.is_staff:
            logger.debug(
                'Permission "%s" granted to user "%s" as superuser or staff',
                self, user
            )
            return True

        if not user.is_authenticated():
            return False

        if Role.objects.filter(groups__user=user, permissions=self).exists():
            return True
        else:
            logger.debug(
                'Fallthru: Permission "%s" not granted to user "%s"', self, user
            )
            return False


@python_2_unicode_compatible
class Role(models.Model):
    """
    This model represents a Role. Roles are permission units. They are the
    only object to which permissions can be granted. They are themselves
    containers too, containing Groups, which are organization units. Roles
    are the basic method to grant a permission to a group. Permissions granted
    to a group using a role, are granted for the entire system.
    """
    label = models.CharField(
        help_text=_('A short text describing the role.'), max_length=128,
        unique=True, verbose_name=_('Label')
    )
    permissions = models.ManyToManyField(
        related_name='roles', to=StoredPermission,
        verbose_name=_('Permissions')
    )
    groups = models.ManyToManyField(
        related_name='roles', to=Group, verbose_name=_('Groups')
    )

    objects = RoleManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Role')
        verbose_name_plural = _('Roles')

    def __str__(self):
        return self.label

    def get_absolute_url(self):
        return reverse(viewname='permissions:role_list')

    def grant(self, permission):
        self.permissions.add(permission.stored_permission)

    def groups_add(self, queryset, _user=None):
        with transaction.atomic():
            event_role_edited.commit(
                actor=_user, target=self
            )
            for obj in queryset:
                self.groups.add(obj)
                event_group_edited.commit(
                    actor=_user, action_object=self, target=obj
                )

    def groups_remove(self, queryset, _user=None):
        with transaction.atomic():
            event_role_edited.commit(
                actor=_user, target=self
            )
            for obj in queryset:
                self.groups.remove(obj)
                event_group_edited.commit(
                    actor=_user, action_object=self, target=obj
                )

    def natural_key(self):
        return (self.label,)
    natural_key.dependencies = ['auth.Group', 'permissions.StoredPermission']

    def permissions_add(self, queryset, _user=None):
        with transaction.atomic():
            event_role_edited.commit(
                actor=_user, target=self
            )
            self.permissions.add(*queryset)

    def permissions_remove(self, queryset, _user=None):
        with transaction.atomic():
            event_role_edited.commit(
                actor=_user, target=self
            )
            self.permissions.remove(*queryset)

    def save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)

        with transaction.atomic():
            is_new = not self.pk
            super(Role, self).save(*args, **kwargs)
            if is_new:
                event_role_created.commit(
                    actor=_user, target=self
                )
            else:
                event_role_edited.commit(
                    actor=_user, target=self
                )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Document indexing'), name='document_indexing'
)

event_index_template_created = namespace.add_event_type(
    label=_('Index created'), name='index_created'
)
event_index_template_edited = namespace.add_event_type(
    label=_('Index edited'), name='index_edited'
)



from __future__ import unicode_literals

from django.template import Library
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

register = Library()


@register.filter
def appearance_get_choice_value(field):
    try:
        return dict(field.field.choices)[field.value()]
    except TypeError:
        return ', '.join([subwidget.data['label'] for subwidget in field.subwidgets if subwidget.data['selected']])
    except KeyError:
        return _('None')


@register.filter
def appearance_get_form_media_js(form):
    return [form.media.absolute_path(path) for path in form.media._js]


@register.simple_tag
def appearance_get_icon(icon_path):
    return import_string(dotted_path=icon_path).render()


@register.simple_tag
def appearance_icon_render(icon_class, enable_shadow=False):
    return icon_class.render(extra_context={'enable_shadow': enable_shadow})



from __future__ import absolute_import, unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from mayan.apps.acls.classes import ModelPermission

from ..models import ErrorLogEntry
from ..permissions_runtime import permission_error_log_view

from .base import GenericViewTestCase
from .literals import TEST_ERROR_LOG_ENTRY_RESULT


class CommonViewTestCase(GenericViewTestCase):
    def _request_about_view(self):
        return self.get(viewname='common:about_view')

    def test_about_view(self):
        response = self._request_about_view()
        self.assertContains(response=response, text='About', status_code=200)

    def _create_error_log_entry(self):
        ModelPermission.register(
            model=get_user_model(), permissions=(permission_error_log_view,)
        )
        ErrorLogEntry.objects.register(model=get_user_model())

        self.error_log_entry = self.test_user.error_logs.create(
            result=TEST_ERROR_LOG_ENTRY_RESULT
        )

    def _request_object_error_log_list(self):
        content_type = ContentType.objects.get_for_model(model=self.test_user)

        return self.get(
            viewname='common:object_error_list', kwargs={
                'app_label': content_type.app_label,
                'model': content_type.model,
                'object_id': self.test_user.pk
            }
        )

    def test_object_error_list_view_no_permissions(self):
        self._create_test_user()
        self._create_error_log_entry()

        response = self._request_object_error_log_list()
        self.assertNotContains(
            response=response, text=TEST_ERROR_LOG_ENTRY_RESULT,
            status_code=403
        )

    def test_object_error_list_view_with_access(self):
        self._create_test_user()
        self._create_error_log_entry()

        self.grant_access(
            obj=self.test_user, permission=permission_error_log_view
        )

        response = self._request_object_error_log_list()
        self.assertContains(
            response=response, text=TEST_ERROR_LOG_ENTRY_RESULT,
            status_code=200
        )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Templating'), name='templating')

permission_template_sandbox = namespace.add_permission(
    label=_('Use the template sandbox'), name='template_sandbox'
)



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('documents', '0043_auto_20180429_0759'),
    ]

    operations = [
        migrations.CreateModel(
            name='FavoriteDocument',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
            ],
            options={
                'verbose_name': 'Favorite document',
                'verbose_name_plural': 'Favorite documents',
            },
        ),
        migrations.AlterModelOptions(
            name='document',
            options={
                'ordering': ('label',), 'verbose_name': 'Document',
                'verbose_name_plural': 'Documents'
            },
        ),
        migrations.AlterField(
            model_name='recentdocument',
            name='document',
            field=models.ForeignKey(
                editable=False, on_delete=django.db.models.deletion.CASCADE,
                related_name='recent', to='documents.Document',
                verbose_name='Document'
            ),
        ),
        migrations.AddField(
            model_name='favoritedocument',
            name='document',
            field=models.ForeignKey(
                editable=False, on_delete=django.db.models.deletion.CASCADE,
                related_name='favorites', to='documents.Document',
                verbose_name='Document'
            ),
        ),
        migrations.AddField(
            model_name='favoritedocument',
            name='user',
            field=models.ForeignKey(
                editable=False, on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL, verbose_name='User'
            ),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon
from mayan.apps.documents.icons import icon_document_type

icon_web_link = Icon(driver_name='fontawesome', symbol='external-link-alt')
icon_document_type_web_links = icon_web_link
icon_document_web_link_list = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)
icon_web_link_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='external-link-alt',
    secondary_symbol='plus'
)
icon_web_link_delete = Icon(driver_name='fontawesome', symbol='times')
icon_web_link_document_types = icon_document_type
icon_web_link_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_web_link_instance_view = Icon(
    driver_name='fontawesome', symbol='external-link-alt'
)
icon_web_link_setup = icon_web_link
icon_web_link_list = icon_web_link



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_document_create_multiple = Icon(
    driver_name='fontawesome', symbol='upload'
)
icon_document_version_upload = Icon(
    driver_name='fontawesome', symbol='upload'
)
icon_log = Icon(driver_name='fontawesome', symbol='exclamation-triangle')
icon_setup_sources = Icon(driver_name='fontawesome', symbol='upload')
icon_setup_source_check_now = Icon(driver_name='fontawesome', symbol='check')
icon_setup_source_delete = Icon(driver_name='fontawesome', symbol='times')
icon_setup_source_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_source_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='upload',
    secondary_symbol='plus'
)
icon_source_list = Icon(driver_name='fontawesome', symbol='upload')
icon_staging_folder = Icon(driver_name='fontawesome', symbol='folder')
icon_staging_folder_file = Icon(driver_name='fontawesome', symbol='file')
icon_staging_file_delete = Icon(driver_name='fontawesome', symbol='times')
icon_upload_view_link = Icon(driver_name='fontawesome', symbol='upload')
icon_wizard_submit = Icon(driver_name='fontawesome', symbol='arrow-right')



from __future__ import unicode_literals

from actstream.models import Action

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..permissions import (
    permission_group_create, permission_group_edit, permission_user_create,
    permission_user_edit
)

from ..events import (
    event_group_created, event_group_edited, event_user_created,
    event_user_edited, event_user_logged_in, event_user_logged_out
)

from .mixins import (
    GroupAPIViewTestMixin, GroupTestMixin, GroupViewTestMixin,
    UserAPIViewTestMixin, UserViewTestMixin
)


class GroupEventsViewTestCase(
    GroupTestMixin, GroupViewTestMixin, GenericViewTestCase
):
    def test_group_create_event(self):
        self.grant_permission(
            permission=permission_group_create
        )
        Action.objects.all().delete()

        response = self._request_test_group_create_view()
        self.assertEqual(response.status_code, 302)

        action = Action.objects.last()

        self.assertEqual(action.actor, self._test_case_user)
        self.assertEqual(action.target, self.test_group)
        self.assertEqual(action.verb, event_group_created.id)

    def test_group_edit_event(self):
        self._create_test_group()
        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )
        Action.objects.all().delete()

        response = self._request_test_group_edit_view()
        self.assertEqual(response.status_code, 302)

        action = Action.objects.last()

        self.assertEqual(action.target, self.test_group)
        self.assertEqual(action.verb, event_group_edited.id)


class GroupEventsAPITestCase(
    GroupAPIViewTestMixin, GroupTestMixin, GroupViewTestMixin,
    BaseAPITestCase
):
    def test_group_create_event_from_api_view(self):
        self.grant_permission(
            permission=permission_group_create
        )
        Action.objects.all().delete()

        response = self._request_test_group_create_api_view()
        self.assertEqual(response.status_code, 201)

        action = Action.objects.last()

        self.assertEqual(action.actor, self.test_group)
        self.assertEqual(action.target, self.test_group)
        self.assertEqual(action.verb, event_group_created.id)

    def test_group_edit_event_from_api_view(self):
        self._create_test_group()

        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )
        Action.objects.all().delete()

        response = self._request_test_group_edit_patch_api_view()
        self.assertEqual(response.status_code, 200)

        action = Action.objects.last()

        self.assertEqual(action.actor, self.test_group)
        self.assertEqual(action.target, self.test_group)
        self.assertEqual(action.verb, event_group_edited.id)


class UserEventsTestCase(GenericViewTestCase):
    auto_login_user = False
    create_test_case_user = False

    def test_user_logged_in_event_from_view(self):
        self._create_test_user()

        Action.objects.all().delete()

        result = self.login(
            username=self.test_user.username,
            password=self.test_user.cleartext_password
        )
        self.assertTrue(result)

        action = Action.objects.order_by('timestamp').last()
        self.assertEqual(action.actor, self.test_user)
        self.assertEqual(action.target, self.test_user)
        self.assertEqual(action.verb, event_user_logged_in.id)

    def test_user_logged_out_event_from_view(self):
        self._create_test_user()

        result = self.login(
            username=self.test_user.username,
            password=self.test_user.cleartext_password
        )
        self.assertTrue(result)

        Action.objects.all().delete()

        self.logout()

        action = Action.objects.order_by('timestamp').last()
        self.assertEqual(action.actor, self.test_user)
        self.assertEqual(action.target, self.test_user)
        self.assertEqual(action.verb, event_user_logged_out.id)


class UserEventsViewTestCase(
    UserAPIViewTestMixin, UserViewTestMixin, GenericViewTestCase
):
    def test_user_create_event_from_view(self):
        self.grant_permission(
            permission=permission_user_create
        )
        Action.objects.all().delete()

        response = self._request_test_user_create_view()
        self.assertEqual(response.status_code, 302)

        action = Action.objects.last()

        self.assertEqual(action.actor, self._test_case_user)
        self.assertEqual(action.target, self.test_user)
        self.assertEqual(action.verb, event_user_created.id)

    def test_user_edit_event_from_view(self):
        self._create_test_user()

        self.grant_access(
            obj=self.test_user, permission=permission_user_edit
        )
        Action.objects.all().delete()

        response = self._request_test_user_edit_view()
        self.assertEqual(response.status_code, 302)

        action = Action.objects.last()

        self.assertEqual(action.actor, self._test_case_user)
        self.assertEqual(action.target, self.test_user)
        self.assertEqual(action.verb, event_user_edited.id)


class UserEventsAPITestCase(
    UserAPIViewTestMixin, UserViewTestMixin, BaseAPITestCase
):
    def test_user_create_event_from_api_view(self):
        self.grant_permission(
            permission=permission_user_create
        )
        Action.objects.all().delete()

        response = self._request_test_user_create_api_view()
        self.assertEqual(response.status_code, 201)

        action = Action.objects.last()

        self.assertEqual(action.actor, self.test_user)
        self.assertEqual(action.target, self.test_user)
        self.assertEqual(action.verb, event_user_created.id)

    def test_user_edit_event_from_api_view(self):
        self._create_test_user()
        self.grant_access(
            obj=self.test_user, permission=permission_user_edit
        )
        Action.objects.all().delete()

        response = self._request_test_user_edit_patch_api_view()
        self.assertEqual(response.status_code, 200)

        action = Action.objects.last()

        self.assertEqual(action.actor, self.test_user)
        self.assertEqual(action.target, self.test_user)
        self.assertEqual(action.verb, event_user_edited.id)



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0009_auto_20150930_2341'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailbasemodel',
            name='store_body',
            field=models.BooleanField(
                default=True,
                help_text='Store the body of the email as a text document.',
                verbose_name='Store email body'
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='imapemail',
            name='mailbox',
            field=models.CharField(
                default='INBOX',
                help_text='IMAP Mailbox from which to check for messages.',
                max_length=64, verbose_name='Mailbox'
            ),
            preserve_default=True,
        ),
    ]



from __future__ import absolute_import, unicode_literals

from ..classes import OCRBackendBase


class NoOpOCR(OCRBackendBase):
    def __init__(self, *args, **kwargs):
        super(NoOpOCR, self).__init__(*args, **kwargs)

    def execute(self, *args, **kwargs):
        """Don't do anything"""



from __future__ import absolute_import, unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.http import Http404
from django.shortcuts import get_object_or_404

from actstream.models import Action, any_stream

from mayan.apps.acls.models import AccessControlList
from mayan.apps.rest_api import generics

from .classes import EventType, EventTypeNamespace
from .models import Notification
from .permissions import permission_events_view
from .serializers import (
    EventSerializer, EventTypeSerializer, EventTypeNamespaceSerializer,
    NotificationSerializer
)


class APIObjectEventListView(generics.ListAPIView):
    """
    get: Return a list of events for the specified object.
    """
    serializer_class = EventSerializer

    def get_object(self):
        content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model']
        )

        try:
            return content_type.get_object_for_this_type(
                pk=self.kwargs['object_id']
            )
        except content_type.model_class().DoesNotExist:
            raise Http404

    def get_queryset(self):
        obj = self.get_object()

        AccessControlList.objects.check_access(
            obj=obj, permissions=(permission_events_view,),
            user=self.request.user
        )

        return any_stream(obj)


class APIEventTypeNamespaceDetailView(generics.RetrieveAPIView):
    """
    get: Returns the details of an event type namespace.
    """
    serializer_class = EventTypeNamespaceSerializer

    def get_object(self):
        try:
            return EventTypeNamespace.get(name=self.kwargs['name'])
        except KeyError:
            raise Http404


class APIEventTypeNamespaceListView(generics.ListAPIView):
    """
    get: Returns a list of all the available event type namespaces.
    """
    serializer_class = EventTypeNamespaceSerializer
    queryset = EventTypeNamespace.all()

    def get_serializer_context(self):
        return {
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }


class APIEventTypeNamespaceEventTypeListView(generics.ListAPIView):
    """
    get: Returns a list of all the available event types from a namespaces.
    """
    serializer_class = EventTypeSerializer

    def get_queryset(self):
        try:
            return EventTypeNamespace.get(
                name=self.kwargs['name']
            ).get_event_types()
        except KeyError:
            raise Http404

    def get_serializer_context(self):
        return {
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }


class APIEventTypeListView(generics.ListAPIView):
    """
    get: Returns a list of all the available event types.
    """
    serializer_class = EventTypeSerializer
    queryset = EventType.all()

    def get_serializer_context(self):
        return {
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }


class APIEventListView(generics.ListAPIView):
    """
    get: Returns a list of all the available events.
    """
    mayan_view_permissions = {'GET': (permission_events_view,)}
    queryset = Action.objects.all()
    serializer_class = EventSerializer

    def get_serializer_context(self):
        return {
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }


class APINotificationListView(generics.ListAPIView):
    """
    get: Return a list of notifications for the current user.
    """
    serializer_class = NotificationSerializer

    def get_queryset(self):
        parameter_read = self.request.GET.get('read')

        if self.request.user.is_authenticated:
            queryset = Notification.objects.filter(user=self.request.user)
        else:
            queryset = Notification.objects.none()

        if parameter_read == 'True':
            queryset = queryset.filter(read=True)
        elif parameter_read == 'False':
            queryset = queryset.filter(read=False)

        return queryset



from __future__ import unicode_literals

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.smart_settings.classes import Setting
from mayan.apps.smart_settings.tests.mixins import SmartSettingTestMixin

from ..settings import setting_shared_storage_arguments


class CommonSettingMigrationTestCase(SmartSettingTestMixin, BaseTestCase):
    def test_common_shared_storage_arguments_0001_migration(self):
        test_value = {'location': 'test value'}
        self.test_setting = setting_shared_storage_arguments
        self.test_config_value = '{}'.format(
            Setting.serialize_value(value=test_value)
        )
        self._create_test_config_file()

        self.assertEqual(
            setting_shared_storage_arguments.value,
            test_value
        )

    def test_common_shared_storage_arguments_0001_migration_with_dict(self):
        test_value = {'location': 'test value'}
        self.test_setting = setting_shared_storage_arguments
        self.test_config_value = test_value
        self._create_test_config_file()

        self.assertEqual(
            setting_shared_storage_arguments.value,
            test_value
        )



from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib import admin

__all__ = ('urlpatterns',)

admin.autodiscover()

urlpatterns = [
    url(regex=r'^admin/', view=admin.site.urls),
]



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import APICommentListView, APICommentView
from .views import (
    DocumentCommentCreateView, DocumentCommentDeleteView,
    DocumentCommentDetailView, DocumentCommentEditView,
    DocumentCommentListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<pk>\d+)/comments/$',
        view=DocumentCommentListView.as_view(), name='comments_for_document'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/comments/add/$',
        view=DocumentCommentCreateView.as_view(), name='comment_add'
    ),
    url(
        regex=r'^comments/(?P<pk>\d+)/delete/$',
        view=DocumentCommentDeleteView.as_view(), name='comment_delete'
    ),
    url(
        regex=r'^comments/(?P<pk>\d+)/$',
        view=DocumentCommentDetailView.as_view(), name='comment_details'
    ),
    url(
        regex=r'^comments/(?P<pk>\d+)/edit/$',
        view=DocumentCommentEditView.as_view(), name='comment_edit'
    ),
]

api_urls = [
    url(
        regex=r'^documents/(?P<document_pk>[0-9]+)/comments/$',
        view=APICommentListView.as_view(), name='comment-list'
    ),
    url(
        regex=r'^documents/(?P<document_pk>[0-9]+)/comments/(?P<comment_pk>[0-9]+)/$',
        view=APICommentView.as_view(), name='comment-detail'
    ),
]



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon
from mayan.apps.documents.icons import icon_document_type

icon_smart_link = Icon(driver_name='fontawesome', symbol='link')

icon_document_type_smart_links = icon_smart_link

icon_smart_link_condition = Icon(driver_name='fontawesome', symbol='code')
icon_smart_link_condition_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='code',
    secondary_symbol='plus'
)
icon_smart_link_condition_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_smart_link_condition_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_smart_link_condition_list = icon_smart_link_condition
icon_smart_link_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='link',
    secondary_symbol='plus'
)
icon_smart_link_delete = Icon(driver_name='fontawesome', symbol='times')
icon_smart_link_document_types = icon_document_type
icon_smart_link_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_smart_link_instances_for_document = Icon(
    driver_name='fontawesome', symbol='link'
)
icon_smart_link_setup = icon_smart_link
icon_smart_link_list = icon_smart_link



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_parsing', '0003_documenttypesettings'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documentpagecontent',
            name='content',
            field=models.TextField(
                blank=True, help_text='The actual text content as extracted '
                'by the document parsing backend.', verbose_name='Content'
            ),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from mayan.apps.documents.tests.base import (
    GenericDocumentViewTestCase, GenericViewTestCase
)

from ..models import Index, IndexInstanceNode
from ..permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_edit,
    permission_document_indexing_instance_view,
    permission_document_indexing_rebuild
)

from .literals import TEST_INDEX_LABEL, TEST_INDEX_LABEL_EDITED
from .mixins import IndexTestMixin, IndexViewTestMixin


class IndexViewTestCase(
    IndexTestMixin, IndexViewTestMixin, GenericViewTestCase
):
    def test_index_create_view_no_permission(self):
        response = self._request_test_index_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Index.objects.count(), 0)

    def test_index_create_view_with_permission(self):
        self.grant_permission(
            permission=permission_document_indexing_create
        )

        response = self._request_test_index_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Index.objects.count(), 1)
        self.assertEqual(Index.objects.first().label, TEST_INDEX_LABEL)

    def test_index_delete_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Index.objects.count(), 1)

    def test_index_delete_view_with_permission(self):
        self._create_test_index()

        self.grant_permission(permission=permission_document_indexing_delete)

        response = self._request_test_index_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Index.objects.count(), 0)

    def test_index_edit_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_index.refresh_from_db()
        self.assertEqual(self.test_index.label, TEST_INDEX_LABEL)

    def test_index_edit_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )

        response = self._request_test_index_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_index.refresh_from_db()
        self.assertEqual(self.test_index.label, TEST_INDEX_LABEL_EDITED)


class IndexInstaceViewTestMixin(object):
    def _request_test_index_instance_node_view(self, index_instance_node):
        return self.get(
            viewname='indexing:index_instance_node_view', kwargs={
                'pk': index_instance_node.pk
            }
        )


class IndexInstaceViewTestCase(
    IndexTestMixin, IndexViewTestMixin, IndexInstaceViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_index_rebuild_view_no_permission(self):
        self.upload_document()
        self._create_test_index()
        self._create_test_index_template_node()

        response = self._request_test_index_rebuild_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(IndexInstanceNode.objects.count(), 1)
        self.assertEqual(IndexInstanceNode.objects.first().parent, None)

    def test_index_rebuild_view_with_access(self):
        self.upload_document()
        self._create_test_index()
        self._create_test_index_template_node()

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_rebuild
        )

        response = self._request_test_index_rebuild_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(IndexInstanceNode.objects.count(), 0)

    def test_index_instance_node_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self.test_index.instance_root
        )
        self.assertEqual(response.status_code, 403)

    def test_index_instance_node_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_instance_view
        )

        response = self._request_test_index_instance_node_view(
            index_instance_node=self.test_index.instance_root
        )
        self.assertContains(response, text=TEST_INDEX_LABEL, status_code=200)


class IndexToolsViewTestMixin(object):
    def _request_indexes_rebuild_get_view(self):
        return self.get(
            viewname='indexing:rebuild_index_instances'
        )

    def _request_indexes_rebuild_post_view(self):
        return self.post(
            viewname='indexing:rebuild_index_instances', data={
                'index_templates': self.test_index.pk
            }
        )

    def _request_indexes_reset_get_view(self):
        return self.get(
            viewname='indexing:index_instances_reset'
        )

    def _request_indexes_reset_post_view(self):
        return self.post(
            viewname='indexing:index_instances_reset', data={
                'index_templates': self.test_index.pk
            }
        )


class IndexToolsViewTestCase(
    IndexTestMixin, IndexViewTestMixin, IndexToolsViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_indexes_rebuild_no_permission(self):
        self._create_test_index(rebuild=False)

        response = self._request_indexes_rebuild_get_view()
        self.assertNotContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_rebuild_post_view()
        # No error since we just don't see the index
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_index.instance_root.get_children_count(), 0
        )

    def test_indexes_rebuild_with_access(self):
        self._create_test_index(rebuild=False)

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_rebuild
        )

        response = self._request_indexes_rebuild_get_view()
        self.assertContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_rebuild_post_view()
        self.assertEqual(response.status_code, 302)

        # An instance root exists
        self.assertTrue(self.test_index.instance_root.pk)

    def test_indexes_reset_no_permission(self):
        self._create_test_index(rebuild=False)
        self._create_test_index_template_node()
        self.test_index.rebuild()

        response = self._request_indexes_reset_get_view()
        self.assertNotContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_reset_post_view()
        # No error since we just don't see the index
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_index.instance_root.get_children_count(), 1
        )

    def test_indexes_reset_with_access(self):
        self._create_test_index(rebuild=False)
        self._create_test_index_template_node()
        self.test_index.rebuild()

        self.grant_access(
            obj=self.test_index,
            permission=permission_document_indexing_rebuild
        )

        response = self._request_indexes_reset_get_view()
        self.assertContains(
            response=response, text=self.test_index.label, status_code=200
        )

        response = self._request_indexes_reset_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_index.instance_root.get_children_count(), 0
        )



from __future__ import absolute_import, unicode_literals

import logging
import os
import sys
import traceback
import warnings

from django import apps
from django.conf import settings
from django.conf.urls import include, url
from django.contrib.auth.signals import user_logged_in
from django.db.models.signals import post_save
from django.utils.encoding import force_text
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from .classes import Template
from .handlers import (
    handler_pre_initial_setup, handler_pre_upgrade,
    handler_user_locale_profile_session_config, handler_user_locale_profile_create
)
from .links import (
    link_about, link_book, link_current_user_locale_profile_edit, link_license,
    link_object_error_list_clear, link_setup, link_tools
)

from .literals import MESSAGE_SQLITE_WARNING
from .menus import menu_about, menu_secondary, menu_topbar, menu_user
from .settings import (
    setting_auto_logging, setting_production_error_log_path,
    setting_production_error_logging
)
from .signals import pre_initial_setup, pre_upgrade
from .tasks import task_delete_stale_uploads  # NOQA - Force task registration
from .utils import check_for_sqlite
from .warnings import DatabaseWarning

logger = logging.getLogger(__name__)


class MayanAppConfig(apps.AppConfig):
    app_namespace = None
    app_url = None

    def ready(self):
        logger.debug('Initializing app: %s', self.name)
        from mayan.urls import urlpatterns as mayan_urlpatterns

        if self.app_url:
            top_url = '{}/'.format(self.app_url)
        elif self.app_url is not None:
            top_url = ''
        else:
            top_url = '{}/'.format(self.name)

        try:
            app_urlpatterns = import_string(
                dotted_path='{}.urls.urlpatterns'.format(self.name)
            )
        except ImportError as exception:
            if force_text(exception) not in ('No module named urls', 'No module named \'{}.urls\''.format(self.name)):
                logger.exception(
                    'Import time error when running AppConfig.ready() of app '
                    '"%s".', self.name
                )
                exc_info = sys.exc_info()
                traceback.print_exception(*exc_info)
                raise exception
        else:
            mayan_urlpatterns += (
                url(
                    regex=r'^{}'.format(top_url), view=include(
                        (app_urlpatterns, self.app_namespace or self.name)
                    )
                ),
            )


class CommonApp(MayanAppConfig):
    app_namespace = 'common'
    app_url = ''
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.common'
    verbose_name = _('Common')

    def ready(self):
        super(CommonApp, self).ready()
        if check_for_sqlite():
            warnings.warn(
                category=DatabaseWarning,
                message=force_text(MESSAGE_SQLITE_WARNING)
            )

        Template(
            name='menu_main', template_name='appearance/menu_main.html'
        )
        Template(
            name='menu_topbar', template_name='appearance/menu_topbar.html'
        )

        menu_user.bind_links(
            links=(
                link_current_user_locale_profile_edit,
            ), position=50
        )

        menu_about.bind_links(
            links=(
                link_tools, link_setup, link_about, link_book, link_license,
            )
        )

        menu_topbar.bind_links(links=(menu_about, menu_user,), position=99)
        menu_secondary.bind_links(
            links=(link_object_error_list_clear,), sources=(
                'common:object_error_list',
            )
        )

        post_save.connect(
            dispatch_uid='common_handler_user_locale_profile_create',
            receiver=handler_user_locale_profile_create,
            sender=settings.AUTH_USER_MODEL
        )
        pre_initial_setup.connect(
            dispatch_uid='common_handler_pre_initial_setup',
            receiver=handler_pre_initial_setup
        )
        pre_upgrade.connect(
            dispatch_uid='common_handler_pre_upgrade',
            receiver=handler_pre_upgrade
        )

        user_logged_in.connect(
            dispatch_uid='common_handler_user_locale_profile_session_config',
            receiver=handler_user_locale_profile_session_config
        )
        self.setup_auto_logging()

    def setup_auto_logging(self):
        if setting_auto_logging.value:
            if settings.DEBUG:
                level = 'DEBUG'
                handlers = ['console']
            else:
                level = 'ERROR'
                handlers = ['console']

            if os.path.exists(settings.MEDIA_ROOT) and setting_production_error_logging.value:
                handlers.append('logfile')

            loggers = {}
            for project_app in apps.apps.get_app_configs():
                loggers[project_app.name] = {
                    'handlers': handlers,
                    'propagate': True,
                    'level': level,
                }

            logging_configuration = {
                'version': 1,
                'disable_existing_loggers': False,
                'formatters': {
                    'intermediate': {
                        '()': 'mayan.apps.common.log.ColorFormatter',
                        'format': '%(name)s <%(process)d> [%(levelname)s] "%(funcName)s() line %(lineno)d %(message)s"',
                    },
                    'logfile': {
                        'format': '%(asctime)s %(name)s <%(process)d> [%(levelname)s] "%(funcName)s() line %(lineno)d %(message)s"'
                    },
                },
                'handlers': {
                    'console': {
                        'class': 'logging.StreamHandler',
                        'formatter': 'intermediate',
                        'level': 'DEBUG',
                    },
                },
                'loggers': loggers
            }

            if os.path.exists(settings.MEDIA_ROOT) and setting_production_error_logging.value:
                logging_configuration['handlers']['logfile'] = {
                    'backupCount': 3,
                    'class': 'logging.handlers.RotatingFileHandler',
                    'filename': setting_production_error_log_path.value,
                    'formatter': 'logfile',
                    'maxBytes': 1024,
                }

            logging.config.dictConfig(logging_configuration)



from __future__ import absolute_import, unicode_literals

import hashlib
import unittest

from fuse import FuseOSError

from django.db import connection

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.models import Document
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.document_indexing.tests.mixins import IndexTestMixin

from ..filesystems import IndexFilesystem

from .literals import (
    TEST_NODE_EXPRESSION, TEST_NODE_EXPRESSION_INVALID,
    TEST_NODE_EXPRESSION_MULTILINE, TEST_NODE_EXPRESSION_MULTILINE_EXPECTED,
    TEST_NODE_EXPRESSION_MULTILINE_2,
    TEST_NODE_EXPRESSION_MULTILINE_2_EXPECTED
)


@unittest.skipIf(connection.vendor == 'mysql', 'Known to fail due to unsupported feature of database manager.')
class IndexFilesystemTestCase(
    IndexTestMixin, DocumentTestMixin, BaseTestCase
):
    auto_upload_document = False

    def test_document_access(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.assertEqual(
            index_filesystem.access(
                '/{}/{}'.format(
                    TEST_NODE_EXPRESSION, self.test_document.label
                )
            ), None
        )

    def test_document_access_failure(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        with self.assertRaises(FuseOSError):
            index_filesystem.access(
                '/{}/{}_non_valid'.format(
                    TEST_NODE_EXPRESSION, self.test_document.label
                )
            )

    def test_document_open(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        file_handle = index_filesystem.open(
            '/{}/{}'.format(TEST_NODE_EXPRESSION, self.test_document.label),
            'rb'
        )

        self.assertEqual(
            hashlib.sha256(
                index_filesystem.read(
                    fh=file_handle, offset=0, path=None,
                    size=self.test_document.size
                )
            ).hexdigest(),
            self.test_document.checksum
        )

    def test_multiline_indexes(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION_MULTILINE,
            link_documents=True
        )

        self.upload_document()
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.assertEqual(
            list(index_filesystem.readdir('/', ''))[2:],
            [TEST_NODE_EXPRESSION_MULTILINE_EXPECTED]
        )

    def test_multiline_indexes_first_and_last(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION_MULTILINE_2,
            link_documents=True
        )

        self.upload_document()
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.assertEqual(
            list(index_filesystem.readdir('/', ''))[2:],
            [TEST_NODE_EXPRESSION_MULTILINE_2_EXPECTED]
        )

    def test_duplicated_indexes(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )
        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.assertTrue(
            'level_1({})'.format(self.test_index.instance_root.get_children().first().pk) in
            list(index_filesystem.readdir('/', ''))
        )

        self.assertTrue(
            'level_1({})'.format(self.test_index.instance_root.get_children().last().pk) in
            list(index_filesystem.readdir('/', ''))
        )

        self.assertEqual(len(list(index_filesystem.readdir('/', ''))), 4)

    def test_ignore_stub_documents(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.test_document = Document.objects.create(
            document_type=self.test_document_type, label='document_stub'
        )
        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)
        self.test_index.rebuild()

        self.assertEqual(
            list(
                index_filesystem.readdir('/', '')
            )[2:], []
        )

    def test_duplicated_documents_readdir(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()
        self.upload_document()

        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.test_index.rebuild()

        self.assertTrue(
            'title_page.png({})'.format(self.test_documents[0].pk) in list(
                index_filesystem.readdir('/level_1', '')
            )
        )
        self.assertTrue(
            'title_page.png({})'.format(self.test_documents[1].pk) in list(
                index_filesystem.readdir('/level_1', '')
            )
        )

    def test_duplicated_documents_open(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()
        self.upload_document()

        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.test_index.rebuild()

        test_document_1_path = '/level_1/title_page.png({})'.format(
            self.test_documents[0].pk
        )
        test_document_2_path = '/level_1/title_page.png({})'.format(
            self.test_documents[1].pk
        )

        file_handle = index_filesystem.open(
            path=test_document_1_path, flags=None
        )

        self.assertEqual(
            index_filesystem.read(
                path=None, size=-1, offset=0, fh=file_handle
            ), self.test_documents[0].open().read()
        )

        index_filesystem.release(path=None, fh=file_handle)

        file_handle = index_filesystem.open(
            path=test_document_2_path, flags=None
        )

        self.assertEqual(
            index_filesystem.read(
                path=None, size=-1, offset=0, fh=file_handle
            ), self.test_documents[1].open().read()
        )

        index_filesystem.release(path=None, fh=file_handle)

    def test_invalid_document_label_character_open(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()

        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.test_document.label = 'Document of 2019/12'
        self.test_document.save()

        self.test_index.rebuild()

        test_document_path = '/level_1/Document of 2019_12'

        file_handle = index_filesystem.open(
            path=test_document_path, flags=None
        )

        self.assertEqual(
            index_filesystem.read(
                path=None, size=-1, offset=0, fh=file_handle
            ), self.test_document.open().read()
        )

        index_filesystem.release(path=None, fh=file_handle)

    def test_invalid_document_label_character_readdir(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION, link_documents=True
        )

        self.upload_document()

        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.test_document.label = 'Document of 2019/12'
        self.test_document.save()

        self.test_index.rebuild()

        self.assertEqual(
            list(
                index_filesystem.readdir('/level_1', '')
            )[2], 'Document of 2019_12'
        )

    def test_invalid_directory_name_character_open(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION_INVALID, link_documents=True
        )

        self.upload_document()

        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.test_index.rebuild()

        test_document_path = '/level_1/{}'.format(self.test_document.label)

        file_handle = index_filesystem.open(
            path=test_document_path, flags=None
        )

        self.assertEqual(
            index_filesystem.read(
                path=None, size=-1, offset=0, fh=file_handle
            ), self.test_document.open().read()
        )

        index_filesystem.release(path=None, fh=file_handle)

    def test_invalid_directory_name_character_readdir(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_NODE_EXPRESSION_INVALID, link_documents=True
        )

        self.upload_document()

        index_filesystem = IndexFilesystem(index_slug=self.test_index.slug)

        self.test_index.rebuild()

        self.assertEqual(
            list(
                index_filesystem.readdir('/', '')
            )[2], 'level_1'
        )

        self.assertEqual(
            list(
                index_filesystem.readdir('/level_1', '')
            )[2], self.test_document.label
        )



from __future__ import absolute_import, unicode_literals

from django.apps import apps
from django.template.loader import render_to_string

from .permissions import permission_document_metadata_view


def widget_document_metadata(context):
    """
    A widget that displays the metadata for the given document
    """
    AccessControlList = apps.get_model(
        app_label='acls', model_name='AccessControlList'
    )
    queryset = AccessControlList.objects.restrict_queryset(
        queryset=context['object'].metadata.all(),
        permission=permission_document_metadata_view,
        user=context['user']
    )

    return render_to_string(
        template_name='metadata/document_metadata_widget.html', context={
            'queryset': queryset
        }
    )



from __future__ import absolute_import, unicode_literals

from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.cache import cache_control, patch_cache_control

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models import Document, DocumentType
from mayan.apps.documents.permissions import permission_document_type_view
from mayan.apps.rest_api import generics

from .literals import WORKFLOW_IMAGE_TASK_TIMEOUT
from .models import Workflow
from .permissions import (
    permission_workflow_create, permission_workflow_delete,
    permission_workflow_edit, permission_workflow_view
)
from .serializers import (
    NewWorkflowDocumentTypeSerializer, WorkflowDocumentTypeSerializer,
    WorkflowInstanceSerializer, WorkflowInstanceLogEntrySerializer,
    WorkflowSerializer, WorkflowStateSerializer, WorkflowTransitionSerializer,
    WritableWorkflowInstanceLogEntrySerializer, WritableWorkflowSerializer,
    WritableWorkflowTransitionSerializer
)

from .settings import settings_workflow_image_cache_time
from .tasks import task_generate_workflow_image


class APIDocumentTypeWorkflowRuntimeProxyListView(generics.ListAPIView):
    """
    get: Returns a list of all the document type workflows.
    """
    mayan_object_permissions = {
        'GET': (permission_workflow_view,),
    }
    serializer_class = WorkflowSerializer

    def get_document_type(self):
        document_type = get_object_or_404(
            klass=DocumentType, pk=self.kwargs['pk']
        )

        AccessControlList.objects.check_access(
            obj=document_type, permissions=(permission_document_type_view,),
            user=self.request.user
        )

        return document_type

    def get_queryset(self):
        return self.get_document_type().workflows.all()


class APIWorkflowDocumentTypeList(generics.ListCreateAPIView):
    """
    get: Returns a list of all the document types attached to a workflow.
    post: Attach a document type to a specified workflow.
    """
    mayan_object_permissions = {
        'GET': (permission_document_type_view,),
    }

    def get_queryset(self):
        """
        This view returns a list of document types that belong to a workflow.
        """
        return self.get_workflow().document_types.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIWorkflowDocumentTypeList, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WorkflowDocumentTypeSerializer
        elif self.request.method == 'POST':
            return NewWorkflowDocumentTypeSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIWorkflowDocumentTypeList, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow': self.get_workflow(),
                }
            )

        return context

    def get_workflow(self):
        """
        Retrieve the parent workflow of the workflow document type.
        Perform custom permission and access check.
        """
        if self.request.method == 'GET':
            permission_required = permission_workflow_view
        else:
            permission_required = permission_workflow_edit

        workflow = get_object_or_404(klass=Workflow, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=workflow, permissions=(permission_required,),
            user=self.request.user
        )

        return workflow


class APIWorkflowDocumentTypeView(generics.RetrieveDestroyAPIView):
    """
    delete: Remove a document type from the selected workflow.
    get: Returns the details of the selected workflow document type.
    """
    lookup_url_kwarg = 'document_type_pk'
    mayan_object_permissions = {
        'GET': (permission_document_type_view,),
    }
    serializer_class = WorkflowDocumentTypeSerializer

    def get_queryset(self):
        return self.get_workflow().document_types.all()

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIWorkflowDocumentTypeView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow': self.get_workflow(),
                }
            )

        return context

    def get_workflow(self):
        """
        This view returns a document types that belongs to a workflow
        RESEARCH: Could the documents.api_views.APIDocumentTypeView class
        be subclasses for this?
        RESEARCH: Since this is a parent-child API view could this be made
        into a generic API class?
        RESEARCH: Reuse get_workflow method from APIWorkflowDocumentTypeList?
        """
        if self.request.method == 'GET':
            permission_required = permission_workflow_view
        else:
            permission_required = permission_workflow_edit

        workflow = get_object_or_404(klass=Workflow, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=workflow, permissions=(permission_required,),
            user=self.request.user
        )

        return workflow

    def perform_destroy(self, instance):
        """
        RESEARCH: Move this kind of methods to the serializer instead it that
        ability becomes available in Django REST framework
        """
        self.get_workflow().document_types.remove(instance)


class APIWorkflowImageView(generics.RetrieveAPIView):
    """
    get: Returns an image representation of the selected workflow.
    """
    mayan_object_permissions = {
        'GET': (permission_workflow_view,),
    }
    queryset = Workflow.objects.all()

    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    @cache_control(private=True)
    def retrieve(self, request, *args, **kwargs):
        task = task_generate_workflow_image.apply_async(
            kwargs=dict(
                document_state_id=self.get_object().pk,
            )
        )

        cache_filename = task.get(timeout=WORKFLOW_IMAGE_TASK_TIMEOUT)
        cache_file = self.get_object().cache_partition.get_file(filename=cache_filename)
        with cache_file.open() as file_object:
            response = HttpResponse(file_object.read(), content_type='image')
            if '_hash' in request.GET:
                patch_cache_control(
                    response,
                    max_age=settings_workflow_image_cache_time.value
                )
            return response


class APIWorkflowRuntimeProxyListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the workflows.
    post: Create a new workflow.
    """
    mayan_object_permissions = {'GET': (permission_workflow_view,)}
    mayan_view_permissions = {'POST': (permission_workflow_create,)}
    queryset = Workflow.objects.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIWorkflowRuntimeProxyListView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WorkflowSerializer
        else:
            return WritableWorkflowSerializer


class APIWorkflowView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected workflow.
    get: Return the details of the selected workflow.
    patch: Edit the selected workflow.
    put: Edit the selected workflow.
    """
    mayan_object_permissions = {
        'DELETE': (permission_workflow_delete,),
        'GET': (permission_workflow_view,),
        'PATCH': (permission_workflow_edit,),
        'PUT': (permission_workflow_edit,)
    }
    queryset = Workflow.objects.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIWorkflowView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WorkflowSerializer
        else:
            return WritableWorkflowSerializer


# Workflow state views


class APIWorkflowStateListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the workflow states.
    post: Create a new workflow state.
    """
    serializer_class = WorkflowStateSerializer

    def get_queryset(self):
        return self.get_workflow().states.all()

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIWorkflowStateListView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow': self.get_workflow(),
                }
            )

        return context

    def get_workflow(self):
        if self.request.method == 'GET':
            permission_required = permission_workflow_view
        else:
            permission_required = permission_workflow_edit

        workflow = get_object_or_404(klass=Workflow, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=workflow, permissions=(permission_required,),
            user=self.request.user
        )

        return workflow


class APIWorkflowStateView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected workflow state.
    get: Return the details of the selected workflow state.
    patch: Edit the selected workflow state.
    put: Edit the selected workflow state.
    """
    lookup_url_kwarg = 'state_pk'
    serializer_class = WorkflowStateSerializer

    def get_queryset(self):
        return self.get_workflow().states.all()

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIWorkflowStateView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow': self.get_workflow(),
                }
            )

        return context

    def get_workflow(self):
        if self.request.method == 'GET':
            permission_required = permission_workflow_view
        else:
            permission_required = permission_workflow_edit

        workflow = get_object_or_404(klass=Workflow, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=workflow, permissions=(permission_required,),
            user=self.request.user
        )

        return workflow


# Workflow transition views


class APIWorkflowTransitionListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the workflow transitions.
    post: Create a new workflow transition.
    """
    def get_queryset(self):
        return self.get_workflow().transitions.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIWorkflowTransitionListView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WorkflowTransitionSerializer
        else:
            return WritableWorkflowTransitionSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIWorkflowTransitionListView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow': self.get_workflow(),
                }
            )

        return context

    def get_workflow(self):
        if self.request.method == 'GET':
            permission_required = permission_workflow_view
        else:
            permission_required = permission_workflow_edit

        workflow = get_object_or_404(klass=Workflow, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=workflow, permissions=(permission_required,),
            user=self.request.user
        )

        return workflow


class APIWorkflowTransitionView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected workflow transition.
    get: Return the details of the selected workflow transition.
    patch: Edit the selected workflow transition.
    put: Edit the selected workflow transition.
    """
    lookup_url_kwarg = 'transition_pk'

    def get_queryset(self):
        return self.get_workflow().transitions.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIWorkflowTransitionView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WorkflowTransitionSerializer
        else:
            return WritableWorkflowTransitionSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIWorkflowTransitionView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow': self.get_workflow(),
                }
            )

        return context

    def get_workflow(self):
        if self.request.method == 'GET':
            permission_required = permission_workflow_view
        else:
            permission_required = permission_workflow_edit

        workflow = get_object_or_404(klass=Workflow, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=workflow, permissions=(permission_required,),
            user=self.request.user
        )

        return workflow


# Document workflow views


class APIWorkflowInstanceListView(generics.ListAPIView):
    """
    get: Returns a list of all the document workflows.
    """
    serializer_class = WorkflowInstanceSerializer
    mayan_object_permissions = {
        'GET': (permission_workflow_view,),
    }

    def get_document(self):
        document = get_object_or_404(klass=Document, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=document, permissions=(permission_workflow_view,),
            user=self.request.user
        )

        return document

    def get_queryset(self):
        return self.get_document().workflows.all()


class APIWorkflowInstanceView(generics.RetrieveAPIView):
    """
    get: Return the details of the selected document workflow.
    """
    lookup_url_kwarg = 'workflow_pk'
    mayan_object_permissions = {
        'GET': (permission_workflow_view,),
    }
    serializer_class = WorkflowInstanceSerializer

    def get_document(self):
        document = get_object_or_404(klass=Document, pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=document, permissions=(permission_workflow_view,),
            user=self.request.user
        )

        return document

    def get_queryset(self):
        return self.get_document().workflows.all()


class APIWorkflowInstanceLogEntryListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the document workflows log entries.
    post: Transition a document workflow by creating a new document workflow log entry.
    """
    def get_document(self):
        document = get_object_or_404(klass=Document, pk=self.kwargs['pk'])

        if self.request.method == 'GET':
            """
            Only test for permission if reading. If writing, the permission
            will be checked in the serializer

            IMPROVEMENT:
            When writing, add check for permission or ACL for the workflow.
            Failing that, check for ACLs for any of the workflow's transitions.
            Failing that, then raise PermissionDenied
            """
            AccessControlList.objects.check_access(
                obj=document, permissions=(permission_workflow_view,),
                user=self.request.user
            )

        return document

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIWorkflowInstanceLogEntryListView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return WorkflowInstanceLogEntrySerializer
        else:
            return WritableWorkflowInstanceLogEntrySerializer

    def get_serializer_context(self):
        context = super(APIWorkflowInstanceLogEntryListView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'workflow_instance': self.get_workflow_instance(),
                }
            )

        return context

    def get_queryset(self):
        return self.get_workflow_instance().log_entries.all()

    def get_workflow_instance(self):
        workflow = get_object_or_404(
            klass=self.get_document().workflows, pk=self.kwargs['workflow_pk']
        )

        return workflow



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_indexing', '0015_auto_20190429_1922'),
    ]

    operations = [
        migrations.AlterField(
            model_name='index',
            name='label',
            field=models.CharField(help_text='Short description of this index.', max_length=128, unique=True, verbose_name='Label'),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.template import Context
from django.urls import reverse

from furl import furl

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.common.tests.literals import TEST_VIEW_NAME
from mayan.apps.permissions import Permission, PermissionNamespace

from ..classes import Link, Menu

from .literals import (
    TEST_PERMISSION_NAMESPACE_NAME, TEST_PERMISSION_NAMESPACE_TEXT,
    TEST_PERMISSION_NAME, TEST_PERMISSION_LABEL, TEST_LINK_TEXT,
    TEST_MENU_NAME, TEST_QUERYSTRING_ONE_KEY, TEST_QUERYSTRING_TWO_KEYS,
    TEST_SUBMENU_NAME, TEST_UNICODE_STRING, TEST_URL
)


class LinkClassTestCase(GenericViewTestCase):
    def setUp(self):
        super(LinkClassTestCase, self).setUp()

        self.test_object = self._test_case_group

        self.add_test_view(test_object=self.test_object)

        self.namespace = PermissionNamespace(
            label=TEST_PERMISSION_NAMESPACE_TEXT,
            name=TEST_PERMISSION_NAMESPACE_NAME
        )

        self.test_permission = self.namespace.add_permission(
            name=TEST_PERMISSION_NAME, label=TEST_PERMISSION_LABEL
        )

        ModelPermission.register(
            model=self.test_object._meta.model,
            permissions=(self.test_permission,)
        )

        self.link = Link(text=TEST_LINK_TEXT, view=TEST_VIEW_NAME)
        Permission.invalidate_cache()

    def test_link_resolve(self):
        response = self.get(viewname=TEST_VIEW_NAME)

        context = Context({'request': response.wsgi_request})

        resolved_link = self.link.resolve(context=context)
        self.assertEqual(
            resolved_link.url, reverse(viewname=TEST_VIEW_NAME)
        )

    def test_link_permission_resolve_no_permission(self):
        link = Link(
            permissions=(self.test_permission,), text=TEST_LINK_TEXT,
            view=TEST_VIEW_NAME
        )

        response = self.get(viewname=TEST_VIEW_NAME)
        response.context.update({'request': response.wsgi_request})

        context = Context(response.context)

        resolved_link = link.resolve(context=context)
        self.assertEqual(resolved_link, None)

    def test_link_permission_resolve_with_permission(self):
        link = Link(
            permissions=(self.test_permission,), text=TEST_LINK_TEXT,
            view=TEST_VIEW_NAME
        )

        self.grant_access(obj=self.test_object, permission=self.test_permission)

        response = self.get(viewname=TEST_VIEW_NAME)
        response.context.update({'request': response.wsgi_request})

        context = Context(response.context)

        resolved_link = link.resolve(context=context)
        self.assertEqual(resolved_link.url, reverse(viewname=TEST_VIEW_NAME))

    def test_link_permission_resolve_with_acl(self):
        # ACL is tested agains the resolved_object or just {{ object }} if not
        link = Link(
            permissions=(self.test_permission,), text=TEST_LINK_TEXT,
            view=TEST_VIEW_NAME
        )

        self.grant_access(obj=self.test_object, permission=self.test_permission)

        response = self.get(viewname=TEST_VIEW_NAME)
        response.context.update({'request': response.wsgi_request})

        context = Context(response.context)

        resolved_link = link.resolve(context=context)
        self.assertNotEqual(resolved_link, None)
        self.assertEqual(resolved_link.url, reverse(viewname=TEST_VIEW_NAME))

    def test_link_with_unicode_querystring_request(self):
        url = furl(reverse(TEST_VIEW_NAME))
        url.args['unicode_key'] = TEST_UNICODE_STRING

        self.link.keep_query = True
        response = self.get(path=url.url)

        context = Context({'request': response.wsgi_request})

        resolved_link = self.link.resolve(context=context)

        self.assertEqual(resolved_link.url, url.url)

    def test_link_with_querystring_preservation(self):
        previous_url = '{}?{}'.format(
            reverse(TEST_VIEW_NAME), TEST_QUERYSTRING_TWO_KEYS
        )
        self.link.keep_query = True
        self.link.url = TEST_URL
        self.link.view = None
        response = self.get(path=previous_url)

        context = Context({'request': response.wsgi_request})

        resolved_link = self.link.resolve(context=context)
        self.assertEqual(
            resolved_link.url,
            '{}?{}'.format(TEST_URL, TEST_QUERYSTRING_TWO_KEYS)
        )

    def test_link_with_querystring_preservation_with_key_removal(self):
        previous_url = '{}?{}'.format(
            reverse(TEST_VIEW_NAME), TEST_QUERYSTRING_TWO_KEYS
        )
        self.link.keep_query = True
        self.link.url = TEST_URL
        self.link.view = None
        self.link.remove_from_query = ['key2']
        response = self.get(path=previous_url)

        context = Context({'request': response.wsgi_request})

        resolved_link = self.link.resolve(context=context)
        self.assertEqual(
            resolved_link.url,
            '{}?{}'.format(TEST_URL, TEST_QUERYSTRING_ONE_KEY)
        )


class MenuClassTestCase(GenericViewTestCase):
    def setUp(self):
        super(MenuClassTestCase, self).setUp()

        self.test_object = self._test_case_group

        self.add_test_view(test_object=self.test_object)

        self.namespace = PermissionNamespace(
            TEST_PERMISSION_NAMESPACE_NAME, TEST_PERMISSION_NAMESPACE_TEXT
        )

        self.test_permission = self.namespace.add_permission(
            name=TEST_PERMISSION_NAME, label=TEST_PERMISSION_LABEL
        )

        self.menu = Menu(name=TEST_MENU_NAME)
        self.sub_menu = Menu(name=TEST_SUBMENU_NAME)
        self.link = Link(text=TEST_LINK_TEXT, view=TEST_VIEW_NAME)
        Permission.invalidate_cache()

    def tearDown(self):
        Menu.remove(name=TEST_MENU_NAME)
        Menu.remove(name=TEST_SUBMENU_NAME)
        super(MenuClassTestCase, self).tearDown()

    def test_null_source_link_unbinding(self):
        self.menu.bind_links(links=(self.link,))

        response = self.get(viewname=TEST_VIEW_NAME)
        context = Context({'request': response.wsgi_request})
        self.assertEqual(
            self.menu.resolve(context=context)[0]['links'][0].link, self.link
        )

        self.menu.unbind_links(links=(self.link,))

        self.assertEqual(self.menu.resolve(context=context), [])

    def test_null_source_submenu_unbinding(self):
        self.menu.bind_links(links=(self.sub_menu,))

        response = self.get(viewname=TEST_VIEW_NAME)
        context = Context({'request': response.wsgi_request})

        self.assertEqual(
            self.menu.resolve(context=context)[0]['links'], [self.sub_menu]
        )

        self.menu.unbind_links(links=(self.sub_menu,))

        self.assertEqual(self.menu.resolve(context=context), [])



from __future__ import unicode_literals

from mayan.apps.rest_api import generics

from .classes import Permission
from .models import Role
from .permissions import (
    permission_role_create, permission_role_delete, permission_role_edit,
    permission_role_view
)
from .serializers import (
    PermissionSerializer, RoleSerializer, WritableRoleSerializer
)


class APIPermissionList(generics.ListAPIView):
    """
    get: Returns a list of all the available permissions.
    """
    serializer_class = PermissionSerializer
    queryset = Permission.all()


class APIRoleListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the roles.
    post: Create a new role.
    """
    mayan_object_permissions = {'GET': (permission_role_view,)}
    mayan_view_permissions = {'POST': (permission_role_create,)}
    queryset = Role.objects.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIRoleListView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return RoleSerializer
        elif self.request.method == 'POST':
            return WritableRoleSerializer


class APIRoleView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected role.
    get: Return the details of the selected role.
    patch: Edit the selected role.
    put: Edit the selected role.
    """
    mayan_object_permissions = {
        'GET': (permission_role_view,),
        'PUT': (permission_role_edit,),
        'PATCH': (permission_role_edit,),
        'DELETE': (permission_role_delete,)
    }
    queryset = Role.objects.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIRoleView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return RoleSerializer
        elif self.request.method in ('PATCH', 'PUT'):
            return WritableRoleSerializer



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_document_submit = Icon(
    driver_name='fontawesome', symbol='chess-board'
)
icon_document_multiple_submit = Icon(
    driver_name='fontawesome', symbol='chess-board'
)
icon_file_metadata = Icon(
    driver_name='fontawesome', symbol='chess-board'
)



from __future__ import unicode_literals

from django.contrib import admin

from .models import StatisticResult


@admin.register(StatisticResult)
class StatisticResultAdmin(admin.ModelAdmin):
    list_display = (
        'slug', 'datetime', 'serialize_data'
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-12-13 00:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0022_auto_20191022_0737'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailbasemodel',
            name='from_metadata_type',
            field=models.ForeignKey(blank=True, help_text='Select a metadata type to store the email\'s "from" value. Must be a valid metadata type for the document type selected previously.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='email_from', to='metadata.MetadataType', verbose_name='From metadata type'),
        ),
        migrations.AlterField(
            model_name='emailbasemodel',
            name='subject_metadata_type',
            field=models.ForeignKey(blank=True, help_text="Select a metadata type to store the email's subject value. Must be a valid metadata type for the document type selected previously.", null=True, on_delete=django.db.models.deletion.CASCADE, related_name='email_subject', to='metadata.MetadataType', verbose_name='Subject metadata type'),
        ),
        migrations.AlterField(
            model_name='source',
            name='label',
            field=models.CharField(db_index=True, help_text='A short text to describe this source.', max_length=128, unique=True, verbose_name='Label'),
        ),
    ]



from __future__ import unicode_literals

from django.conf.urls import url

from .views import (
    CheckVersionView, DependencyGroupEntryListView,
    DependencyGroupEntryDetailView, DependencyGroupListView,
    DependencyLicensesView
)

urlpatterns = [
    url(
        regex=r'^check_version/$', view=CheckVersionView.as_view(),
        name='check_version_view'
    ),
    url(
        regex=r'^groups/(?P<dependency_group_name>\w+)/(?P<dependency_group_entry_name>\w+)$',
        view=DependencyGroupEntryDetailView.as_view(),
        name='dependency_group_entry_detail'
    ),
    url(
        regex=r'^groups/(?P<dependency_group_name>\w+)/$',
        view=DependencyGroupEntryListView.as_view(),
        name='dependency_group_entry_list'
    ),
    url(
        regex=r'^groups/$', view=DependencyGroupListView.as_view(),
        name='dependency_group_list'
    ),
    url(
        regex=r'^licenses/$', view=DependencyLicensesView.as_view(),
        name='dependency_licenses_view'
    ),
]



from __future__ import unicode_literals

from django.apps import apps

from .literals import WORKFLOW_IMAGE_CACHE_NAME


def callback_update_workflow_image_cache_size(setting):
    Cache = apps.get_model(app_label='file_caching', model_name='Cache')
    cache = Cache.objects.get(name=WORKFLOW_IMAGE_CACHE_NAME)
    cache.maximum_size = setting.value
    cache.save()



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Web links'), name='linking'
)

event_web_link_created = namespace.add_event_type(
    label=_('Web link created'), name='web_link_created'
)
event_web_link_edited = namespace.add_event_type(
    label=_('Web link edited'), name='web_link_edited'
)
event_web_link_navigated = namespace.add_event_type(
    label=_('Web link navigated'), name='web_link_navigated'
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

# Don't import anything on star import, we just want to make it easy
# for apps.py to activate the settings in this module.
__all__ = ()
namespace = Namespace(label=_('Celery'), name='celery')

setting_celery_broker_url = namespace.add_setting(
    global_name='CELERY_BROKER_URL', default=None,
    help_text=_(
        'Default: "amqp://". Default broker URL. This must be a URL in '
        'the form of: transport://userid:password@hostname:port/virtual_host '
        'Only the scheme part (transport://) is required, the rest is '
        'optional, and defaults to the specific transports default values.'
    ),
)
setting_celery_result_backend = namespace.add_setting(
    global_name='CELERY_RESULT_BACKEND', default=None,
    help_text=_(
        'Default: No result backend enabled by default. The backend used '
        'to store task results (tombstones). Refer to '
        'http://docs.celeryproject.org/en/v4.1.0/userguide/configuration.'
        'html#result-backend'
    )
)



from __future__ import unicode_literals

from mayan.apps.smart_settings.classes import NamespaceMigration
from mayan.apps.smart_settings.utils import smart_yaml_load


class FileMetadataSettingMigration(NamespaceMigration):
    """
    From version 0001 to 0002 backend arguments are no longer quoted
    but YAML valid too. Changed in version 3.3.
    """
    def file_metadata_drivers_arguments_0001(self, value):
        return smart_yaml_load(value=value)



from __future__ import unicode_literals

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..models import LayerTransformation

from .mixins import TransformationTestMixin, TransformationViewsTestMixin


class TransformationViewsTestCase(
    TransformationTestMixin, TransformationViewsTestMixin,
    GenericDocumentViewTestCase
):
    def test_transformation_create_view_no_permission(self):
        transformation_count = LayerTransformation.objects.count()

        response = self._request_transformation_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            LayerTransformation.objects.count(), transformation_count
        )

    def test_transformation_create_view_with_permission(self):
        self.grant_access(
            obj=self.test_document, permission=self.test_permission
        )

        transformation_count = LayerTransformation.objects.count()

        response = self._request_transformation_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            LayerTransformation.objects.count(), transformation_count + 1
        )

    def test_transformation_delete_view_no_permission(self):
        self._create_test_transformation()

        transformation_count = LayerTransformation.objects.count()

        response = self._request_transformation_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            LayerTransformation.objects.count(), transformation_count
        )

    def test_transformation_delete_view_with_access(self):
        self._create_test_transformation()

        self.grant_access(
            obj=self.test_document, permission=self.test_layer_permission
        )

        transformation_count = LayerTransformation.objects.count()

        response = self._request_transformation_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            LayerTransformation.objects.count(), transformation_count - 1
        )

    def test_transformation_edit_view_no_permission(self):
        self._create_test_transformation()

        transformation_arguments = self.test_transformation.arguments

        response = self._request_transformation_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_transformation.refresh_from_db()
        self.assertEqual(
            transformation_arguments, self.test_transformation.arguments
        )

    def test_transformation_edit_view_with_access(self):
        self._create_test_transformation()

        self.grant_access(
            obj=self.test_document, permission=self.test_layer_permission
        )

        transformation_arguments = self.test_transformation.arguments
        response = self._request_transformation_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_transformation.refresh_from_db()
        self.assertNotEqual(
            transformation_arguments, self.test_transformation.arguments
        )

    def test_transformation_list_view_no_permission(self):
        self._create_test_transformation()

        response = self._request_transformation_list_view()
        self.assertNotContains(
            response=response, text=self.test_document.label, status_code=404
        )
        self.assertNotContains(
            response=response,
            text=self.test_transformation.get_transformation_class().label,
            status_code=404
        )

    def test_transformation_list_view_with_access(self):
        self._create_test_transformation()

        self.grant_access(
            obj=self.test_document, permission=self.test_permission
        )

        response = self._request_transformation_list_view()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )
        self.assertContains(
            response=response,
            text=self.test_transformation.get_transformation_class().label,
            status_code=200
        )



from __future__ import unicode_literals

from ..models import WebLink

from .literals import (
    TEST_WEB_LINK_LABEL, TEST_WEB_LINK_LABEL_EDITED, TEST_WEB_LINK_TEMPLATE
)


class WebLinkTestMixin(object):
    def _create_test_web_link(self):
        self.test_web_link = WebLink.objects.create(
            label=TEST_WEB_LINK_LABEL, template=TEST_WEB_LINK_TEMPLATE,
        )


class WebLinkViewTestMixin(object):
    def _request_test_document_web_link_instance_view(self):
        return self.post(
            viewname='web_links:web_link_instance_view', kwargs={
                'document_pk': self.test_document.pk,
                'web_link_pk': self.test_web_link.pk
            }
        )

    def _request_test_document_web_link_list_view(self):
        return self.get(
            viewname='web_links:document_web_link_list', kwargs={
                'pk': self.test_document.pk
            }
        )

    def _request_test_web_link_create_view(self):
        return self.post(
            viewname='web_links:web_link_create', data={
                'label': TEST_WEB_LINK_LABEL,
                'template_template': TEST_WEB_LINK_TEMPLATE,
            }
        )

    def _request_test_web_link_delete_view(self):
        return self.post(
            viewname='web_links:web_link_delete', kwargs={
                'pk': self.test_web_link.pk
            }
        )

    def _request_test_web_link_edit_view(self):
        return self.post(
            viewname='web_links:web_link_edit', kwargs={
                'pk': self.test_web_link.pk
            }, data={
                'label': TEST_WEB_LINK_LABEL_EDITED,
                'template_template': TEST_WEB_LINK_TEMPLATE
            }
        )

    def _request_test_web_link_list_view(self):
        return self.get(viewname='web_links:web_link_list')



from __future__ import unicode_literals

from django import apps
from django.test.runner import DiscoverRunner

from .literals import EXCLUDE_TEST_TAG


class MayanTestRunner(DiscoverRunner):
    @classmethod
    def add_arguments(cls, parser):
        DiscoverRunner.add_arguments(parser)
        parser.add_argument(
            '--mayan-apps', action='store_true', default=False,
            dest='mayan_apps',
            help='Test all Mayan apps that report to have tests.'
        )
        parser.add_argument(
            '--no-exclude', action='store_true', default=False,
            dest='no_exclude',
            help='Include excluded tests.'
        )

    def __init__(self, *args, **kwargs):
        self.mayan_apps = kwargs.pop('mayan_apps')
        self.no_exclude = kwargs.pop('no_exclude')
        super(MayanTestRunner, self).__init__(*args, **kwargs)

        # Test that should be excluded by default
        # To include then pass --tag=exclude to the test runner invocation
        if not self.no_exclude:
            if EXCLUDE_TEST_TAG not in self.tags:
                self.exclude_tags |= set((EXCLUDE_TEST_TAG,))

    def build_suite(self, *args, **kwargs):
        # Apps that report they have tests
        if self.mayan_apps:
            args = list(args)
            args[0] = [
                app.name for app in apps.apps.get_app_configs() if getattr(
                    app, 'has_tests', False
                )
            ]

        return super(MayanTestRunner, self).build_suite(*args, **kwargs)



from __future__ import unicode_literals

from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.sources.links import link_document_version_upload

from ..literals import STATE_CHECKED_OUT, STATE_LABELS
from ..models import DocumentCheckout
from ..permissions import (
    permission_document_check_in, permission_document_check_in_override,
    permission_document_check_out, permission_document_check_out_detail_view
)

from .mixins import DocumentCheckoutTestMixin, DocumentCheckoutViewTestMixin


class DocumentCheckoutViewTestCase(
    DocumentCheckoutTestMixin, DocumentCheckoutViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_check_in_get_view_no_permission(self):
        self._check_out_test_document()

        response = self._request_test_document_check_in_get_view()
        self.assertNotContains(
            response=response, text=self.test_document.label, status_code=404
        )

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_in_get_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self.test_document, permission=permission_document_check_in
        )

        response = self._request_test_document_check_in_get_view()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_in_post_view_no_permission(self):
        self._check_out_test_document()

        response = self._request_test_document_check_in_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_in_post_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self.test_document, permission=permission_document_check_in
        )

        response = self._request_test_document_check_in_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(self.test_document.is_checked_out())
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_document
            )
        )

    def test_document_multiple_check_in_post_view_no_permission(self):
        # Upload second document
        self.upload_document()

        self._check_out_test_document(document=self.test_documents[0])
        self._check_out_test_document(document=self.test_documents[1])

        response = self._request_test_document_multiple_check_in_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(self.test_documents[0].is_checked_out())
        self.assertTrue(self.test_documents[1].is_checked_out())
        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[0]
            )
        )
        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[1]
            )
        )

    def test_document_multiple_check_in_post_view_with_document_0_access(self):
        # Upload second document
        self.upload_document()

        self._check_out_test_document(document=self.test_documents[0])
        self._check_out_test_document(document=self.test_documents[1])

        self.grant_access(
            obj=self.test_documents[0], permission=permission_document_check_in
        )

        response = self._request_test_document_multiple_check_in_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(self.test_documents[0].is_checked_out())
        self.assertTrue(self.test_documents[1].is_checked_out())
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[0]
            )
        )
        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[1]
            )
        )

    def test_document_multiple_check_in_post_view_with_access(self):
        # Upload second document
        self.upload_document()

        self._check_out_test_document(document=self.test_documents[0])
        self._check_out_test_document(document=self.test_documents[1])

        self.grant_access(
            obj=self.test_documents[0], permission=permission_document_check_in
        )
        self.grant_access(
            obj=self.test_documents[1], permission=permission_document_check_in
        )

        response = self._request_test_document_multiple_check_in_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(self.test_documents[0].is_checked_out())
        self.assertFalse(self.test_documents[1].is_checked_out())
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[0]
            )
        )
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[1]
            )
        )

    def test_document_check_out_get_view_no_permission(self):
        response = self._request_test_document_check_out_get_view()
        self.assertEqual(response.status_code, 404)

        self.assertFalse(self.test_document.is_checked_out())

    def test_document_check_out_get_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_check_out
        )

        response = self._request_test_document_check_out_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertFalse(self.test_document.is_checked_out())

    def test_document_check_out_post_view_no_permission(self):
        response = self._request_test_document_check_out_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertFalse(self.test_document.is_checked_out())

    def test_document_check_out_post_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_check_out
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_check_out_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_multiple_check_out_post_view_no_permission(self):
        # Upload second document
        self.upload_document()

        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_check_out_detail_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_multiple_check_out_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertFalse(self.test_documents[0].is_checked_out())
        self.assertFalse(self.test_documents[1].is_checked_out())
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[0]
            )
        )
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[1]
            )
        )

    def test_document_multiple_check_out_post_view_with_document_access(self):
        # Upload second document
        self.upload_document()

        self.grant_access(
            obj=self.test_documents[0], permission=permission_document_check_out
        )
        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_check_out_detail_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_multiple_check_out_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(self.test_documents[0].is_checked_out())
        self.assertFalse(self.test_documents[1].is_checked_out())
        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[0]
            )
        )
        self.assertFalse(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[1]
            )
        )

    def test_document_multiple_check_out_post_view_with_access(self):
        # Upload second document
        self.upload_document()

        self.grant_access(
            obj=self.test_documents[0], permission=permission_document_check_out
        )
        self.grant_access(
            obj=self.test_documents[1], permission=permission_document_check_out
        )
        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_check_out_detail_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_multiple_check_out_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(self.test_documents[0].is_checked_out())
        self.assertTrue(self.test_documents[1].is_checked_out())
        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[0]
            )
        )
        self.assertTrue(
            DocumentCheckout.objects.is_checked_out(
                document=self.test_documents[1]
            )
        )

    def test_document_check_out_detail_view_no_permission(self):
        self._check_out_test_document()

        response = self._request_test_document_check_out_detail_view()

        self.assertNotContains(
            response, text=STATE_LABELS[STATE_CHECKED_OUT], status_code=404
        )

    def test_document_check_out_detail_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )

        response = self._request_test_document_check_out_detail_view()
        self.assertContains(
            response, text=STATE_LABELS[STATE_CHECKED_OUT], status_code=200
        )

    def test_document_check_out_list_view_no_permission(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_view
        )

        response = self._request_test_document_check_out_list_view()
        self.assertNotContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_check_out_list_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_out_detail_view
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_view
        )

        response = self._request_test_document_check_out_list_view()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_check_in_forcefull_view_no_permission(self):
        # Gitlab issue #237
        # Forcefully checking in a document by a user without adequate
        # permissions throws out an error
        self._create_test_user()
        # Check out document as test_user
        self._check_out_test_document(user=self.test_user)

        self.grant_access(
            obj=self.test_document, permission=permission_document_check_in
        )

        response = self.post(
            viewname='checkouts:check_in_document', kwargs={
                'pk': self.test_document.pk
            }
        )
        self.assertEqual(response.status_code, 302)

        self.assertTrue(self.test_document.is_checked_out())

    def test_document_check_in_forcefull_view_with_access(self):
        self._create_test_user()
        # Check out document as test_user
        self._check_out_test_document(user=self.test_user)

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_check_in_override
        )

        # Check in document as test_case_user
        response = self.post(
            viewname='checkouts:check_in_document', kwargs={
                'pk': self.test_document.pk
            }
        )
        self.assertEqual(response.status_code, 302)

        self.assertFalse(self.test_document.is_checked_out())


class NewVersionBlockViewTestCase(
    DocumentCheckoutTestMixin, DocumentCheckoutViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_check_out_block_new_version(self):
        """
        Gitlab issue #231
        User shown option to upload new version of a document even though it
        is blocked by checkout - v2.0.0b2

        Expected results:
            - Link to upload version view should not resolve
            - Upload version view should reject request
        """
        self._create_test_case_superuser()
        self._check_out_test_document()
        self.login_superuser()

        response = self.post(
            viewname='sources:document_version_upload', kwargs={
                'document_pk': self.test_document.pk
            }, follow=True
        )

        self.assertContains(
            response=response, text='blocked from uploading',
            status_code=200
        )

        response = self.get(
            viewname='documents:document_version_list', kwargs={
                'pk': self.test_document.pk
            }, follow=True
        )

        # Needed by the url view resolver
        response.context.current_app = None
        resolved_link = link_document_version_upload.resolve(
            context=response.context
        )

        self.assertEqual(resolved_link, None)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.dependencies.classes import BinaryDependency

from .settings import setting_pdftotext_path

BinaryDependency(
    help_text=_(
        'Utility from the poppler-utils package used to text content '
        'from PDF files.'
    ), label='PDF to text', module=__name__, name='pdftotext',
    path=setting_pdftotext_path.value
)



from __future__ import unicode_literals

from ..permissions import permission_document_view
from ..search import document_search, document_page_search

from .base import GenericDocumentViewTestCase


class DocumentSearchTestMixin(object):
    def _perform_document_page_search(self):
        return document_page_search.search(
            query_string={'q': self.test_document.label}, user=self._test_case_user
        )

    def _perform_document_search(self):
        return document_search.search(
            query_string={'q': self.test_document.label}, user=self._test_case_user
        )


class DocumentSearchTestCase(
    DocumentSearchTestMixin, GenericDocumentViewTestCase
):
    def test_document_page_search_no_access(self):
        queryset = self._perform_document_page_search()
        self.assertFalse(self.test_document.pages.first() in queryset)

    def test_document_page_search_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        queryset = self._perform_document_page_search()
        self.assertTrue(self.test_document.pages.first() in queryset)

    def test_document_search_no_access(self):
        queryset = self._perform_document_search()
        self.assertFalse(self.test_document in queryset)

    def test_document_search_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        queryset = self._perform_document_search()
        self.assertTrue(self.test_document in queryset)
